# -*- coding: utf-8 -*-
#
# ----------------------------------------------------- #
#                                                       #
#  This code is a part of T-matrix fitting project      #
#  Contributors:                                        #
#   L. Avakyan <laavakyan@sfedu.ru>                     #
#   D. Kostyulin <kostyulin@sfedu.ru>                   #
#                                                       #
# ----------------------------------------------------- #
"""
Contributions to optical extinction spectra from axial-symmetric
particles. Currently, spheroids.
"""
from __future__ import print_function
from __future__ import division
import numpy as np
try:
    import matplotlib.pyplot as plt
except ImportError:
    pass

# use input in both python2 and python3
try:
    input = raw_input
except NameError:
    pass
# use xrange in both python2 and python3
try:
    xrange
except NameError:
    xrange = range

try:
    from scatterpy.tmatrix import calc_T  # , calc_T_inner
    from scatterpy.shapes import spheroid
except ImportError:
    print('WARNING: Could not load `scatterpy` library!')
    print('Spheroid functional will be disabled')
    pass

from mstm_studio.contributions import MieSingleSphere


class SpheroidSP(MieSingleSphere):
    """
    Extinction from spheroid calculated in T-matrix approach
    using external library `ScatterPy`
    <https://github.com/TCvanLeth/ScatterPy>
    """
    number_of_params = 3
    NORDER = 5   # number of harmonics
    NGAUSS = 11  # integration points

    def calculate(self, values):
        """
        Parameters:

            values: list of parameters `scale`, `size` and `aspect`
                    Scale is an arbitrary multiplier.
                    Size parameter is the radius of equivelent-volume
                    sphere.
                    The aspect ratio is
                    "the ratio of horizontal to rotational axes"
                    according to scatterpy/shapes.py

        Return:

            extinction efficiency array for spheroid particle
        """
        self._check(values)
        if self.material is None:
            raise Exception('T-matrix calculation requires material data. Stop.')
        Cext = np.zeros(len(self.wavelengths))
        if calc_T is None:  # failed to import scatterpy
            return Cext
        nk = self.material.get_n(self.wavelengths) + \
             1j * self.material.get_k(self.wavelengths)
        for iwl, wl in enumerate(self.wavelengths):
            # print('SpheroidSP: current wavelength %.0f nm' % wl)
            size_param = 2 * np.abs(values[1]) * self.matrix
            T = calc_T(size_param, wl, nk[iwl] / self.matrix,  # rtol=0.001,
                       n_maxorder=self.NORDER, n_gauss=self.NGAUSS,
                       sfunc=lambda x: spheroid(np.array([np.abs(values[2])])))
            Nmax = T.shape[-3]
            for n in range(1, Nmax+1):
                Cext[iwl] += np.real(T[0, 0, n-1, n-1, 0, 0] +
                                     T[0, 0, n-1, n-1, 1, 1])
                for m in range(1, n+1):
                    Cext[iwl] += 2 * np.real(T[0, m, n-1, n-1, 0, 0] +
                                             T[0, m, n-1, n-1, 1, 1])
        Cext = -self.wavelengths**2 / (2 * np.pi) * Cext
        Cext = Cext / (np.pi * size_param**2 / 4.0)
        return values[0] * Cext

    def plot_shape(self, values, fig=None, axs=None):
        """
        Plot shape profile.
        Spatial shape is achieved by rotation over vertical axis.

        Parameters:

            values: list of control parameters
                    `scale`, `size` and `aspect`

            fig: matplotlib figure

            axs: matplotlib axes

        Return:

            filled/created fig and axs objects
        """
        flag = fig is None
        if flag:
            fig = plt.figure()
            axs = fig.add_subplot(111)
        theta = np.linspace(0, 2*np.pi, 100)
        # 4/3 pi size^3 = 4/3 pi a^2 c
        # aspect = a / c
        a = values[1] * values[2]**(1/3.)
        c = a / values[2]
        # oblate / prolate speroid surface function from [Tsang1984]
        r = 1 / np.sqrt((np.sin(theta)/a)**2 + (np.cos(theta)/c)**2)
        x = r * np.sin(theta)
        z = r * np.cos(theta)
        axs.plot(x, z, 'b')
        axs.plot(-x, z, 'b')
        axs.plot([0, 0], [np.min(z), np.max(z)], 'b--')
        axs.set_aspect('equal', adjustable='box')
        axs.set_xlabel('X, nm')
        axs.set_ylabel('Z, nm')
        if flag:
            plt.show()
        return fig, axs




# -*- coding: utf-8 -*-
#
# ----------------------------------------------------- #
#                                                       #
#  This code is a part of T-matrix fitting project      #
#  Contributors:                                        #
#   L. Avakyan <laavakyan@sfedu.ru>                     #
#   K. Yablunovskiy <kirill-yablunovskii@mail.ru>       #
#                                                       #
# ----------------------------------------------------- #
"""
Based on heaviliy rewritten MSTM-GUI code
<URL:https://github.com/dmayerich/mstm-gui>
<https://git.stim.ee.uh.edu/optics/mstm-gui.git>
by Dr. David Mayerich

Optimized for spectral calculations (for many wavelengths)
in order to use for fitting to experiment
"""
from __future__ import print_function
from __future__ import division
import numpy as np
from numpy.random import lognormal
from scipy import interpolate
import subprocess
import os   # to delete files after calc.
import sys  # to check whether running on Linux or Windows
import datetime
import time
import tempfile  # to run mstm in temporary directory
try:
    import matplotlib.pyplot as plt
except ImportError:
    pass
# use input in both python2 and python3
try:
    input = raw_input
except NameError:
    pass
# use xrange in both python2 and python3
try:
    xrange
except NameError:
    xrange = range


class Profiler(object):
    '''
    This class for benchmarking is from
    http://onesteptospace.blogspot.pt/2013/01/python.html
    Usage:
    >>> with Profiler() as p:
    >>>     // your code to be profiled here
    '''
    def __enter__(self):
        self._startTime = time.time()

    def __exit__(self, type, value, traceback):
        print('Elapsed time: {:.3f} sec'.format(time.time() - self._startTime))


class SpheresOverlapError(Exception):
    pass


class SPR(object):
    '''
    Class for calculation of surface plasmin resonance (SPR),
    running MSTM external code.
    The MSTM executable should be set in MSTM_BIN environment
    variable. Default is ~/bin/mstm.x
    '''

    environment_material = 'Air'

    paramDict = {
      'number_spheres': 0,
      'sphere_position_file': '',          # radius, X,Y,Z [nm], n ,k
      'length_scale_factor': 1.0,          # 2π/λ[nm]
      'real_ref_index_scale_factor': 1.0,  # multiplier for spheres
      'imag_ref_index_scale_factor': 1.0,
      'real_chiral_factor': 0.0,        # chiral passive spheres
      'imag_chiral_factor': 0.0,
      'medium_real_ref_index': 1.0,     # refraction index of the environment
      'medium_imag_ref_index': 0.0,
      'medium_real_chiral_factor': 0.0,
      'medium_imag_chiral_factor': 0.0,
      'target_euler_angles_deg': [0.0, 0.0, 0.0],  # ignored for random orient. calc.

      'mie_epsilon': 1.0E-12,           # Convergence criterion for determining the number of orders
                                        # in the Mie expansions. Negative value - number of orders.
      'translation_epsilon': 1.0E-8,    # Convergence criterion for estimating the maximum order of the cluster T matrix
      'solution_epsilon': 1.0E-8,       # Precision of linear equation system solution
      't_matrix_convergence_epsilon': 1.0E-6,
      'plane_wave_epsilon': 1E-3,       # Precision of expansion of incedent field (both for palne and gaussian waves)
      'iterations_per_correction': 20,  # ignored for big 'near_field_translation_distance'
      'max_number_iterations': 2000,    # with account of all iterations
      'near_field_translation_distance': 1.0E6,  # can be big real, small real or negative. TWEAK FOR PERFORMANCE
      'store_translation_matrix': 0,
      'fixed_or_random_orientation': 1,  # 0 - fixed, 1 - random
      'gaussian_beam_constant': 0,       # CB = 1/(k ω0). CB = 0 - plane wave
      'gaussian_beam_focal_point': [0.0, 0.0, 0.0],  # does not alters results for plane wave and random orientations
      'run_print_file': '',              # if balnk will use stdout
      'write_sphere_data': 0,            # 1 - detail, 0 - concise

      'output_file': 'test.dat',         # should change for each run

      'incident_or_target_frame': 0,     # used for scattering matrix output
      'min_scattering_angle_deg': 0.0,
      'max_scattering_angle_deg': 180.0,
      'min_scattering_plane_angle_deg': 0.0,   # selects a plane for fixed orient.
      'max_scattering_plane_angle_deg': 0.0,   # selects a plane for fixed orient.
      'delta_scattering_angle_deg': 1.0,
      'calculate_near_field': 0,       # no near field calculations
      'calculate_t_matrix': 1,         # 1 - new calc., 0 - use old, 2 - continue calc
      't_matrix_file': 'tmatrix-temp.dat',
      'sm_number_processors': 10,      # actual number of procesors is
                                       # minimum to this value and provided by mpi
    }

    local_keys = ['output_file', 'length_scale_factor',
                  'medium_real_ref_index', 'medium_imag_ref_index',
                  't_matrix_file']

    def __init__(self, wavelengths):
        '''
        Parameter:
            wavelengths: numpy array
                Wavelegths in nm
        '''
        self.wavelengths = wavelengths
        self.command = os.environ.get('MSTM_BIN', '~/bin/mstm.x')

    def set_spheres(self, spheres):
        self.spheres = spheres
        # count spheres with positive radius:
        self.paramDict['number_spheres'] = np.sum(self.spheres.a > 0)

    def simulate(self, outfn=None):
        '''
        Start the simulation.

        The inpuit parameters are read from object dictionary `paramDict`.
        Routine will prepare input file `scriptParams.inp` in the temporary folder,
        which will be deleted after calculation.

        After calculation the result depends on the polarization setting.
        For polarized light the object fields will be filled:

            extinction_par, extinction_ort,
            absorbtion_par, absorbtion_ort,
            scattering_par, scattering_ort.

        While for orientation-averaged calculation just:

            extinction, absorbtion and scattering.
        '''
        if self.paramDict['number_spheres'] == 0:  # np spheres
            return self.wavelengths, np.zeros_like(self.wavelengths)
        if self.spheres.check_overlap():
            raise SpheresOverlapError('Spheres overlapping!')
        if isinstance(self.environment_material, Material):
            material = self.environment_material
        else:
            print(self.environment_material)
            material = Material(self.environment_material)
        with tempfile.TemporaryDirectory() as tmpdir:
            print('Using temporary directory: %s' % tmpdir)
            outFID = open(os.path.join(tmpdir, 'scriptParams.inp'), 'w')
            outFID.write('begin_comment\n')
            outFID.write('**********************************\n')
            outFID.write('  MSTM input for SPR calculation\n')
            outFID.write('  Generated by python script\n')
            outFID.write('  %s\n' %
                         datetime.datetime.now().strftime('%Y-%m-%d %H:%M'))
            outFID.write('**********************************\n')
            outFID.write('end_comment\n')
            for key in self.paramDict.keys():
                if key not in self.local_keys:
                    outFID.write(key + '\n')
                    if isinstance(self.paramDict[key], str):
                        svalue = self.paramDict[key]
                    else:
                        if isinstance(self.paramDict[key], list):
                            svalue = '  '.join(map(str, self.paramDict[key]))
                        else:
                            svalue = str(self.paramDict[key])
                        # replace exponent symbol
                        svalue = svalue.replace('e', 'd', 1)
                    outFID.write('%s \n' % svalue)

            for l in self.wavelengths:
                outFID.write('begin_comment\n')
                outFID.write('**********************************\n')
                outFID.write('  Wavelength  %.3f \n' % l)
                outFID.write('**********************************\n')
                outFID.write('end_comment\n')
                outFID.write('output_file\n')
                outFID.write('mstm_l%.0f.out\n' % (l * 1000))
                outFID.write('length_scale_factor\n')
                outFID.write('  %.6f\n' % (2.0 * 3.14159 / l))
                outFID.write('medium_real_ref_index\n')
                outFID.write('  %f\n' % material.get_n(l))
                outFID.write('medium_imag_ref_index\n')
                outFID.write('  %f\n' % material.get_k(l))

                outFID.write('sphere_sizes_and_positions\n')

                for i in xrange(len(self.spheres)):
                    a = self.spheres.a[i]
                    if a > 0:  # consider only positive radii
                        x = self.spheres.x[i]
                        y = self.spheres.y[i]
                        z = self.spheres.z[i]
                        self.spheres.materials[i].D = 2 * a
                        n = self.spheres.materials[i].get_n(l)
                        k = self.spheres.materials[i].get_k(l)
                        outFID.write('  %.4f  %.4f  %.4f  %.4f  %.3f  %.3f \n' %
                                     (a, x, y, z, n, k))
                outFID.write('new_run\n')

            outFID.write('end_of_options\n')
            outFID.close()

            # run the binary
            if sys.platform == 'win32':
                si = subprocess.STARTUPINFO()
                si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
                subprocess.call('%s scriptParams.inp > NUL' % self.command,
                                shell=True, startupinfo=si, cwd=tmpdir)
            else:
                subprocess.call('%s scriptParams.inp > /dev/null' % self.command,
                                shell=True, cwd=tmpdir)

            # parse the simulation results
            if self.paramDict['fixed_or_random_orientation'] == 0:  # fixed orientation
                self.extinction_par = []  # parallel polarization (\hat \alpha)
                self.absorbtion_par = []
                self.scattering_par = []
                self.extinction_ort = []  # perpendicular polarization (\hat \beta)
                self.absorbtion_ort = []
                self.scattering_ort = []
                for l in self.wavelengths:
                    inFID = open(os.path.join(tmpdir,
                                              'mstm_l%.0f.out' % (l * 1000)),
                                 'r')
                    while True:
                        line = inFID.readline()
                        if 'scattering matrix elements' in line:
                            break
                        elif 'parallel total ext, abs, scat efficiencies' in line:
                            values = map(float,
                                         inFID.readline().strip().split())
                            values = list(values)
                            self.extinction_par.append(float(values[0]))
                            self.absorbtion_par.append(float(values[1]))
                            self.scattering_par.append(float(values[2]))
                        elif 'perpendicular total ext' in line:
                            values = map(float,
                                         inFID.readline().strip().split())
                            values = list(values)
                            self.extinction_ort.append(float(values[0]))
                            self.absorbtion_ort.append(float(values[1]))
                            self.scattering_ort.append(float(values[2]))
                    inFID.close()
                    os.remove(os.path.join(tmpdir,
                                           'mstm_l%.0f.out' % (l * 1000)))
                self.extinction_par = np.array(self.extinction_par)
                self.absorbtion_par = np.array(self.absorbtion_par)
                self.scattering_par = np.array(self.scattering_par)
                self.extinction_ort = np.array(self.extinction_ort)
                self.absorbtion_ort = np.array(self.absorbtion_ort)
                self.scattering_ort = np.array(self.scattering_ort)
                return (self.wavelengths,
                        (self.extinction_par + self.extinction_ort))
            else:    # random orientation
                self.extinction = []
                self.absorbtion = []
                self.scattering = []
                for lam in self.wavelengths:
                    fnl = os.path.join(tmpdir, 'mstm_l%.0f.out' % (lam * 1000))
                    with open(fnl, 'r') as fout:
                        while True:
                            line = fout.readline()
                            if 'scattering matrix elements' in line:
                                break
                            elif 'total ext, abs, scat efficiencies' in line:
                                values = map(float,
                                             fout.readline().strip().split())
                                values = list(values)  # python3 is evil
                                self.extinction.append(float(values[0]))
                                self.absorbtion.append(float(values[1]))
                                self.scattering.append(float(values[2]))
                    os.remove(fnl)
                self.extinction = np.array(self.extinction)
                self.absorbtion = np.array(self.absorbtion)
                self.scattering = np.array(self.scattering)
            if outfn is not None:
                self.write(outfn)
        return self.wavelengths, self.extinction


    def plot(self):
        '''
        Plot results with matplotlib.pyplot
        '''
        plt.plot(self.wavelengths, self.extinction, 'r-', label='extinction')
        plt.show()
        return plt

    def write(self, filename):
        '''
        Save results to file
        '''
        if self.paramDict["fixed_or_random_orientation"] == 1:  # random
            fout = open(filename, 'w')
            fout.write('#Wavel.\tExtinct.\n')
            for i in range(len(self.wavelengths)):
                fout.write('%.4f\t%.8f\r\n' % (self.wavelengths[i],
                                               self.extinction[i]))
            fout.close()
        else:   # fixed
            fout = open(filename, 'w')
            fout.write('#Wavel.\tExt_par\tExt_ort\n')
            for i in range(len(self.wavelengths)):
                fout.write('%.4f\t%.8f\t%.8f\r\n' % (self.wavelengths[i],
                           self.extinction_par[i], self.extinction_ort[i]))
            fout.close()

    def set_incident_field(self, fixed=False, azimuth_angle=0.0,
                           polar_angle=0.0, polarization_angle=0.0):
        '''
            Set incident wave orientation and polarization

            Parameters:

                fixed: bool
                    True  - fixed orientation and polarized light
                    False - average over all orientations and polarizations

                azimuth_angle, polar_angle: float (degrees)

                polarization_angle: float (degrees)
                    polarization angle relative to the `k-z` palne.
                    0 - X-polarized, 90 - Y-polarized (if `azimuth` and
                    `polar` angles are zero).
        '''
        if not fixed:
            self.paramDict['fixed_or_random_orientation'] = 1  # random
        else:
            self.paramDict['fixed_or_random_orientation'] = 0  # fixed
            self.paramDict['incident_azimuth_angle_deg'] = azimuth_angle
            self.paramDict['incident_polar_angle_deg'] = polar_angle
            self.paramDict['polarization_angle_deg'] = polarization_angle


class Material(object):
    r"""
    Material class.

    Use `get_n()` and `get_k()` methods to obtain values of refraction
    index at arbitraty wavelength (in nm).
    """
    def __init__(self, file_name, wls=None, nk=None, eps=None):
        r"""
        Parameters:

        file_name:
            1. complex value, written in numpy format or as string;
            2. one of the predefined strings (air, water, glass);
            3. filename with optical constants.

            File header should state `lambda`, `n` and `k` columns
            If either `nk= n + 1j*k` or `eps = re + 1j*im` arrays are
            specified, then the data from one of them will be used
            and filename content will be ignored.

        wls: float array
            array of wavelengths (in nm) used for data interpolation.
            If None then ``np.linspace(300, 800, 500)`` will be used.

        """
        if isinstance(file_name, str):
            self.__name__ = 'Mat_%s' % os.path.basename(file_name)
        else:
            self.__name__ = 'Mat_%.3f' % file_name

        if wls is None:
            wl_min = 200   # 149.9
            wl_max = 1200  # 950.1
            wls = np.array([wl_min, wl_max])
        k = np.array([0.0, 0.0])
        if nk is not None:
            n = np.real(nk)
            k = np.imag(nk)
        elif eps is not None:
            mod = np.absolute(eps)
            n = np.sqrt((mod + np.real(eps)) / 2)
            k = np.sqrt((mod - np.real(eps)) / 2)
        else:
            try:
                np.complex(file_name)
                is_complex = True
            except ValueError:
                is_complex = False
            if is_complex:
                nk = np.complex(file_name)
                n = np.array([np.real(nk), np.real(nk)])
                k = np.array([np.imag(nk), np.imag(nk)])
            else:
                if file_name.lower() == 'air':
                    n = np.array([1.0, 1.0])
                elif file_name.lower() == 'water':
                    n = np.array([1.33, 1.33])
                elif file_name.lower() == 'glass':
                    n = np.array([1.66, 1.66])
                else:
                    optical_constants = np.genfromtxt(file_name, names=True)
                    wls = optical_constants['lambda']
                    if np.max(wls) < 100:  # wavelengths are in micrometers
                        wls = wls * 1000   # convert to nm
                    n = optical_constants['n']
                    k = optical_constants['k']
                    if wls[0] > wls[1]:  # form bigger to smaller
                        wls = np.flipud(wls)  # reverse order
                        n = np.flipud(n)
                        k = np.flipud(k)
                    n = n[wls > wl_min]
                    k = k[wls > wl_min]
                    wls = wls[wls > wl_min]
                    n = n[wls < wl_max]
                    k = k[wls < wl_max]
                    wls = wls[wls < wl_max]
        wl_step = np.abs(wls[1] - wls[0])
        if (wl_step > 1.1) and (wl_step < 500):
            interp_kind = 'cubic'                # cubic interpolation
        else:  # too dense or too sparse mesh, linear interpolation is needed
            interp_kind = 'linear'
        # print('Interpolation kind : %s'%interp_kind)
        self._get_n_interp = interpolate.interp1d(wls, n, kind=interp_kind)
        self._get_k_interp = interpolate.interp1d(wls, k, kind=interp_kind)

    def get_n(self, wl):
        return self._get_n_interp(wl)

    def get_k(self, wl):
        return self._get_k_interp(wl)

    def __str__(self):
        return self.__name__

    def plot(self, wls=None, fig=None, axs=None):
        r"""
        plot ``n`` and ``k`` dependence from wavelength

        Parameters:

            wls: float array
                array of wavelengths (in nm). If None then
                ``np.linspace(300, 800, 500)`` will be used.

            fig: matplotlib figure

            axs: matplotlib axes

        Return:

            filled/created fig and axs objects
        """
        if wls is None:
            wls = np.linspace(300, 800, 500)
        flag = fig is None
        if flag:
            fig = plt.figure()
            axs = fig.add_subplot(111)
        axs.plot(wls, self.get_n(wls), label='Real')
        axs.plot(wls, self.get_k(wls), label='Imag')
        axs.set_ylabel('Refraction index')
        axs.set_xlabel('Wavelength, nm')
        axs.legend()
        if flag:
            plt.show()
        return fig, axs


# class MaterialManager():
    # """
    # Cache for materials, to decrease file i/o
    # """
    # def __init__(self, wavelengths):
        # self.materials = {}


class Spheres(object):
    """
    Abstract collection of spheres

    Object fields:
        N: int
            number of spheres
        x, y, z: numpy arrays
            coordinates of spheres centers
        a: list or arrray
            spheres radii
        materials: numpy array
            Material objects or strings
    """
    def __init__(self):
        """
        Creates empty collection of spheres. Use child classes for non-empty!
        """
        self.N = 0
        self.x = []
        self.y = []
        self.z = []
        self.a = []  # radius
        self.materials = []

    def __len__(self):
        return self.N

    def check_overlap(self, eps=0.001):
        """
        Check if spheres are overlapping
        """
        result = False
        n = len(self.x)
        for i in xrange(n):
            for j in xrange(i + 1, n):
                dx = abs(self.x[j] - self.x[i])
                dy = abs(self.y[j] - self.y[i])
                dz = abs(self.z[j] - self.z[i])
                Ri = self.a[i]
                Rj = self.a[j]
                dist = np.sqrt(dx * dx + dy * dy + dz * dz)
                if dist < Ri + Rj + eps:
                    # distance between spheres is less than sum of thier radii
                    # but there still can be nested spheres, check it
                    if Ri > Rj:
                        result = Ri < dist + Rj + eps
                    else:  # Rj < Ri
                        result = Rj < dist + Ri + eps
                if result:  # avoid unneeded steps
                    return True
        return result

    def append(self, sphere):
        """
        Append by data from SingleSphere object

        Parameter:

            sphere: SingleSphere
        """
        self.a = np.append(self.a, sphere.a[0])
        self.x = np.append(self.x, sphere.x[0])
        self.y = np.append(self.y, sphere.y[0])
        self.z = np.append(self.z, sphere.z[0])
        self.materials.append(sphere.materials[0])
        self.N += 1

    def delete(self, i):
        """
        Delete element with index `i`
        """
        self.a = np.delete(self.a, i)
        self.x = np.delete(self.x, i)
        self.y = np.delete(self.y, i)
        self.z = np.delete(self.z, i)
        self.materials.pop(i)
        self.N -= 1

    def extend(self, spheres):
        """
        Append by all items from object `spheres`
        """
        for i in xrange(len(spheres)):
            self.append(SingleSphere(spheres.x[i], spheres.y[i],
                        spheres.z[i], spheres.a[i], spheres.materials[i]))

    def get_center(self, method=''):
        """
        calculate center of masses in assumption of uniform density

        Parameter:

            method: string {''|'mass'}
                If method == 'mass' then center of masses
                (strictly speaking, volumes) is calculated.
                Otherwise all spheres are averaged evenly.
        """
        weights = np.ones(self.N)
        if method.lower() == 'mass':
            weights = self.a**3
        Xc = np.sum(np.dot(self.x, weights)) / np.sum(weights)
        Yc = np.sum(np.dot(self.y, weights)) / np.sum(weights)
        Zc = np.sum(np.dot(self.z, weights)) / np.sum(weights)
        return np.array((Xc, Yc, Zc))

    def load(self, filename, mat_filename='etaGold.txt', units='nm'):
        """
            Reads spheres coordinates and radii from file.

            Parameters:

                filename: string
                    file to be read from

                mat_filename: string
                    all spheres will have this material (sphere-material
                    storaging is not yet implemented)

                units: string {'mum'|'nm'}
                    distance units.
                    If 'mum' then coordinated will be scaled (x1000)
        """
        x = []
        y = []
        z = []
        a = []
        try:
            f = open(filename, 'r')
            text = f.readlines()
            for line in text:
                if line[0] != '#':  # skip comment and header
                    words = [w.strip() for w in line.replace(',', '.').split()]
                    data = [float(w) for w in words]
                    a.append(data[0])
                    x.append(data[1])
                    y.append(data[2])
                    z.append(data[3])
            f.close()
        except Exception as err:
            print('Load failed \n %s' % err)
        self.N = len(a)
        self.x = np.array(x)
        self.y = np.array(y)
        self.z = np.array(z)
        self.a = np.array(a)
        if units == 'mum':
            self.x = self.x * 1000.0
            self.y = self.y * 1000.0
            self.z = self.z * 1000.0
            self.a = self.a * 1000.0
        self._set_material(mat_filename)

    def save(self, filename):
        """
        Saves spheres coordinates and radii to file.

        Parameter:

            filename: string
        """
        try:
            f = open(filename, 'w')
            f.write('#radius\tx\ty\tz\tn\tk\r\n')
            for i in xrange(self.N):
                wl = 555
                a = self.a[i]
                x = self.x[i]
                y = self.y[i]
                z = self.z[i]
                n = self.materials[i].get_n(wl)
                k = self.materials[i].get_k(wl)
                f.write('%f\t\t%f\t\t%f\t\t%f\t\t%f\t\t%f\r\n' %
                        (a, x, y, z, n, k))
        except Exception as err:
            print('Save failed \n %s' % err)
        finally:
            f.close()


class SingleSphere(Spheres):
    """
    Collection of spheres with only one sphere
    """
    def __init__(self, x, y, z, a, mat_filename='etaGold.txt'):
        """
        Parameters:

            x, y, z: float
                coordinates of spheres centers

            a: float
                spheres radii

            mat_filename: string, float, complex value or Material object
                material specification
        """
        self.N = 1
        self.x = np.array([x])
        self.y = np.array([y])
        self.z = np.array([z])
        self.a = np.array([a])
        if isinstance(mat_filename, Material):
            self.materials = [mat_filename]
        else:
            self.materials = [Material(mat_filename)]


class LogNormalSpheres(Spheres):
    """
    The set of spheres positioned on the regular mesh
    with random Log-Normal distributed sizes.
    In the case overlapping of the spheres the sizes
    should(?) be regenerated.
    """
    def __init__(self, N, mu, sigma, d, mat_filename='etaGold.txt'):
        """
        Parameters:

            N: int
                number of spheres
            mu, sigma: floats
                parameters of Log-Normal distribution
            d: float
                average empty space between spheres centers
            mat_filename: string or Material object
                specification of spheres material
        """
        # estimate the box size:
        a = mu  # average sphere radius
        A = (N**(1. / 3) + 1) * (d + 2 * a)
        print('Box size estimated as: %.1f nm' % A)
        # A = A*1.5
        Xc = []
        Yc = []
        Zc = []
        x = -A / 2.0
        while x < A / 2.0:
            y = -A / 2.0
            while y < A / 2.0:
                z = -A / 2.0
                while z < A / 2.0:
                    if (x * x + y * y + z * z < A * A / 4.0):
                        Xc.append(x)
                        Yc.append(y)
                        Zc.append(z)
                    z = z + (2 * a + d)
                y = y + (2 * a + d)
            x = x + (2 * a + d)
        print('Desired number of particles: %i' % N)
        print('Number of particles in a box: %i' % len(Xc))
        self.N = min([N, len(Xc)])
        print('Resulted number of particles: %i' % self.N)
        self.x = np.array(Xc)
        self.y = np.array(Yc)
        self.z = np.array(Zc)
        random_a = lognormal(np.log(mu), sigma, self.N)  # nm
        random_a = random_a
        self.a = np.array(random_a)
        if isinstance(mat_filename, Material):
            mat = mat_filename
        else:
            mat = Material(mat_filename)
        self.materials = [mat for i in xrange(self.N)]


class ExplicitSpheres (Spheres):
    def __init__(self, N=0, Xc=[], Yc=[], Zc=[], a=[],
                 mat_filename='etaGold.txt'):
        """
        Create explicitely defined spheres

        Parameters:
            N: int
                number of spheres
            Xc, Yc, Zc: lists or numpy arrays
                coordinates of the spheres centers
            a: list or numpy array
                radii of the spheres
            mat_filename: string, list of strings, Material or list of
                Materials specification of spheres material

            Note: If only first array Xc is supplied, than all data is
            assumed zipped in it,
            i.e.: `Xc = [X1, Y1, Z1, a1, ..., XN, YN, ZN, aN]`
        """
        super(ExplicitSpheres, self).__init__()
        self.N = N
        if N == 0:  # special case of empty object
            self.x = []
            self.y = []
            self.z = []
            self.a = []
            return
        if N < len(Xc):  # data is zipped in Xc
            assert(4 * N == len(Xc))
            self.x = np.zeros(N)
            self.y = np.zeros(N)
            self.z = np.zeros(N)
            self.a = np.zeros(N)
            i = 0
            while i < len(Xc):
                self.x[i // 4] = Xc[i + 0]
                self.y[i // 4] = Xc[i + 1]
                self.z[i // 4] = Xc[i + 2]
                self.a[i // 4] = abs(Xc[i + 3])
                i = i + 4
        else:
            self.x = np.array(Xc)
            self.y = np.array(Yc)
            self.z = np.array(Zc)
            self.a = np.abs(np.array(a))

        if isinstance(mat_filename, (Material, str)):
            # one material filename for all spheres
            self._set_material(mat_filename)
        elif isinstance(mat_filename, list):
            # list of material filenames for all spheres
            if len(mat_filename) == 1:
                self._set_material(mat_filename[0])
            else:
                assert(len(mat_filename) == self.N)
                for mat_fn in mat_filename:
                    # TODO: use material manager to avoid re-creating
                    # and extra file reads
                    if isinstance(mat_fn, Material):
                        self.materials.append(mat_fn)
                    else:
                        self.materials.append(Material(mat_fn))
        else:
            raise Exception('Bad material variable: %s' % str(mat_filename))

        # if self.check_overlap():
            # print('Warning: Spheres are overlapping!')

    def _set_material(self, mat_filename):
        if isinstance(mat_filename, Material):
            mat = mat_filename
        else:
            mat = Material(mat_filename)
        self.materials = [mat for i in xrange(self.N)]


if __name__ == '__main__':
    print('Overlap tests')
    spheres = Spheres()
    print('  Test not overlapped... ')
    spheres.x = [-5, 5]
    spheres.y = [0, 0]
    spheres.z = [0, 0]
    spheres.a = [4, 4]
    assert(not spheres.check_overlap())
    print('  Test overlapped... ')
    spheres.a = [5, 5]
    assert(spheres.check_overlap())
    print('  Test nested... ')
    spheres.x = [0, 0]
    spheres.a = [2, 5]
    assert(not spheres.check_overlap())
    spheres.a = [5, 3]
    assert(not spheres.check_overlap())
    # input('Press enter')

    print('Materials test')
    mat = Material(os.path.join('nk', 'etaGold.txt'))
    # mat.plot()
    mat1 = Material(os.path.join('nk', 'etaSilver.txt'))
    mat3 = Material('glass')
    mat5 = Material(1.5)
    mat6 = Material('2.0+0.5j')
    mat7 = Material('mat7', wls=np.linspace(300, 800, 100),
                    nk=np.linspace(-10, 5, 100) + 1j * np.linspace(0, 10, 100))
    mat8 = Material('mat7', wls=np.linspace(300, 800, 100),
                    eps=np.linspace(-10, 5, 100) + 1j * np.linspace(0, 10, 100))
    print('etaGold ', mat.get_n(800))
    print('etaSilver ', mat1.get_n(800))
    print('Glass (constant) ', mat3.get_n(800), mat3.get_k(800))
    print('n=1.5 material ', mat5.get_n(550))
    print('n=2.0+0.5j material ', mat6.get_n(550), mat6.get_k(550))
    print('nk material ', mat7.get_n(550), mat7.get_k(550))
    print('eps material ', mat8.get_n(550), mat8.get_k(550))
    # input('Press enter')
    with Profiler() as p:
        wls = np.linspace(300, 800, 100)
        # create SPR object
        spr = SPR(wls)
        spr.environment_material = 'glass'
        # spr.set_spheres(SingleSphere(0.0, 0.0, 0.0, 25.0, 'etaGold.txt'))
        spheres = ExplicitSpheres(2, [0, 0, 0, 10, 0, 0, 0, 12],
                                  mat_filename=['nk/etaGold.txt',
                                                'nk/etaSilver.txt'])
        # spheres = ExplicitSpheres(2, [0,0,0,20,0,0,0,21],
        #                           mat_filename='etaGold.txt')
        spr.set_spheres(spheres)
        # spr.set_spheres(LogNormalSpheres(27, 0.020, 0.9, 0.050 ))
        # calculate!
        # spr.command = ''
        spr.simulate()
    spr.plot()
    input('Press enter')



# -*- coding: utf-8 -*-
#-----------------------------------------------------#
#                                                     #
# This code is a part of T-matrix fitting project     #
# Contributors:                                       #
#  L. Avakyan <laavakyan@sfedu.ru>                    #
#  A. Skidanenko <ann.skidanenko@ya.ru>               #
#                                                     #
#-----------------------------------------------------#
"""
  Fitting of particle aggreagate T-matrix spectrum
  to experimental SPR spectrum.
"""
from __future__ import print_function
import os
from mstm_studio.mstm_spectrum import SPR, ExplicitSpheres, SpheresOverlapError
from mstm_studio.contributions import ConstantBackground
import numpy as np
from scipy import interpolate
import scipy.optimize as so
from datetime import datetime
import threading

try:
    import matplotlib.pyplot as plt
except:
    pass

# use input in both python2 and python3
try:
   input = raw_input
except NameError:
   pass
# use xrange in both python2 and python3
try:
    xrange
except NameError:
    xrange = range

class Parameter(object):
    """
    Class for parameter object used for storage of
    parameter's name, value and variation limits.

    Parameter naming conventions:

        `scale` - outer common multiplier

        `ext%i` - extra parameter, like background, peaks or Mie contributions

        `a%i` - sphere radius

        `x%i`, `y%i`, `z%i` - coordinates of sphere center

    where `%i` is a number (0, 1, 2, ...)
    """
    def __init__(self, name, value=1, min=None, max=None, internal_loop=False):
        """
        Parameters:

            name: string
                name of parameter used for constraints etc

            value: float
                initial value of parameter

            min, max: float
                bounds for parameter variation (optional)

            internal_loop : bool
                if `True` the parameter will be allowed to vary in internal
                (fast) loop, which does not require MSTM recalculation.
                Note: this flag will be removed in future.

            varied: bool
                if `True` -- will be changed during fit
        """
        self.name = name
        self.value = self.ini_value = value
        self.min = min
        self.max = max
        self.internal_loop = internal_loop
        self.varied = True
        # ...

    def __str__(self):
        return '%s : %f' % (self.name, self.value)


class Constraint(object):
    """
    Abstract constraint class. All other should inherit from it.
    """
    def apply(self, params):
        """
        Modify the params dict
        according to given constranint algorithm.

        Note: Abstract method!
        """
        pass


class FixConstraint(Constraint):
    def __init__(self, prm, value=None):
        """
        Fix value of parameter with name `prm` to `value`.

        Parameters:

            prm: string
                parameter name

            value: float
                if `None` than initial value will be used.
        """
        self.prm = prm.lower()
        self.value = value

    def apply(self, params):
        """ Apply fix constraint """
        assert self.prm in params
        if self.value is not None:
            params[self.prm].value = self.value
        params[self.prm].varied = False


class EqualityConstraint(Constraint):
    def __init__(self, prm1, prm2):
        """
        Fix two parameters with names `prm1` and `prm2` being equal
        """
        self.prm1 = prm1.lower()
        self.prm2 = prm2.lower()

    def apply(self, params):
        """ Apply equality constraint """
        assert self.prm1 in params
        assert self.prm2 in params
        params[self.prm2].value = params[self.prm1].value
        params[self.prm2].varied = False


class ConcentricConstraint(Constraint):
    def __init__(self, i1, i2):
        """
        Two spheres with common centers.

        `i1` and `i2` -- indexes of spheres
        """
        self.constraints = [EqualityConstraint('x%02i'%i1, 'x%02i'%i2),
                            EqualityConstraint('y%02i'%i1, 'y%02i'%i2),
                            EqualityConstraint('z%02i'%i1, 'z%02i'%i2)]

    def apply(self, params):
        """ Apply concentric constraint """
        for c in self.constraints:
            c.apply(params)


class RatioConstraint(Constraint):
    def __init__(self, prm1, prm2, ratio=1):
        """
        Maintain ratio of two variables, `prm1`/`prm2` = `ratio`
        """
        self.prm1 = prm1.lower()
        self.prm2 = prm2.lower()
        self.set_ratio(ratio)

    def apply(self, params):
        """ Apply Ratio constraint """
        assert self.prm1 in params
        assert self.prm2 in params
        params[self.prm2].value = params[self.prm1].value / self.ratio
        params[self.prm2].varied = False

    def set_ratio(self, ratio):
        """
        Set ratio of :math:`prm1/prm2 = ratio`.
        """
        assert np.abs(ratio) > 1e-10
        self.ratio = ratio


class Fitter(threading.Thread):
    """
    Class to perform fit of experimental Exctinction spectrum

    Field:

        tolerance: float
            stopping criterion, default is 1e-4
    """

    tolerance = 1e-4  # stopping criterion

    def __init__(self, exp_filename, wl_min=300, wl_max=800, wl_npoints=51,
                 extra_contributions=None, plot_progress=False):
        """
        Parameters:

            exp_filename: str
                name of file with experimental data

            wl_min, wl_max: float
                wavelength bounds for fitting (in nm).

            wl_npoints: int
                number of wavelengths where spectra will be calcualted and compared.

            extra_contributions: list of Contribution objects
                If `None`, then ConstantBackground will be used.
                Assuming that first element is a background.
                If you don't want any extra contribution, set to empty list `[]`.

            plot_progress: bool
                Show fitting progress using matplotlib.
                Should be turned off when run on parallel cluster without gui.
        """
        super(Fitter, self).__init__()
        self._stop_event = threading.Event()  # to be able to stop outside

        self.exp_filename = exp_filename
        data = np.loadtxt(self.exp_filename)    # load data
        data = data[np.argsort(data[:,0]),:]    # sort by 0th column
        if np.max(data[:,0]) < 10:      # if values are really low
            print('WARNING: Data X column is probably in mum, automatilcally rescaling to nm.')
            data[:,0] = data[:,0] * 1000

        self.wl_min = max(wl_min, data[ 0, 0])
        self.wl_max = min(wl_max, data[-1, 0])
        self.wl_npoints = wl_npoints
        print('Wavelength limits are setted to: %f < wl < %f'% (self.wl_min, self.wl_max))
        self.wls, self.exp = self._rebin(self.wl_min, self.wl_max, self.wl_npoints,
                                         data[:,0], data[:,1])
        self.params = {}             # dictionaty of parameters objects
        self.spheres = None          # object of Spheres
        self.constraints = []        # list of Constraint objects
        self.calc = np.zeros_like(self.wls)  # calculated spectrum
        self.chisq = -1              # chi square (squared residual)
        # set scale as default
        self.set_scale()
        # add extra contributions
        self.extra_contributions = []
        self.set_extra_contributions(extra_contributions)
        # set matrix material as default
        self.set_matrix()
        # plot, if specified
        self.plot_progress = plot_progress
        if self.plot_progress:
            plt.ion()
            self.fig = plt.figure()
            ax = self.fig.add_subplot(111)
            ax.plot(self.wls, self.exp, 'ro')
            self.line1, = ax.plot(self.wls, self.calc, 'b-')
            self.fig.canvas.draw()
            #~ self.lock = threading.Lock()  # used to sync with main thread where plot
        # callback function supplied outside
        self._cbuser = None

    def _rebin(self, xmin, xmax, N, x, y):
        """
        hidden method used to rebin data to uniform scale
        """
        f = interpolate.interp1d(x, y)
        xnew = np.linspace(xmin, xmax, N)
        ynew = f(xnew)
        return xnew, ynew

    def _print_params(self):
        for key in sorted(self.params):
            print('params[ %s ] \t %s ' % (key, self.params[key]))

    def set_matrix(self, material='AIR'):
        """
        set refraction index of matrix material

        material : {'AIR'|'WATER'|'GLASS'} or float
            the name of material or
            refraction index value.
        """
        self.MATRIX_MATERIAL = material

    def set_scale(self, value=1):
        if 'scale' not in self.params:
            self.params['scale'] = Parameter('scale', value=value, internal_loop=True)
        else:
            self.params['scale'].value = value
            self.params['scale'].ini_value = value

    def set_extra_contributions(self, contributions, initial_values=None):
        """
        Add extra contributions and initialize corresponding params.

        Parameters:

            contributions: list of Contribution objests

            initial_values: float array
        """
        # remove old parameters
        i_tot = 0
        for contribution in self.extra_contributions:
            if contribution is not None:
                n = contribution.number_of_params
                for _ in range(n):
                    self.params.pop('ext%02i' % i_tot)
                    i_tot += 1

        if contributions is None:
            contributions = [ConstantBackground(self.wls, 'ConstBkg')]
        self.extra_contributions = contributions[:]

        # create new parameter objects
        n_tot = 0
        i_tot = 0
        for contribution in self.extra_contributions:
            n = contribution.number_of_params
            for _ in range(n):
                self.params['ext%02i' % i_tot] = Parameter('ext%02i' % i_tot, value=0.1, internal_loop=True)
                i_tot += 1
            n_tot += n
        self.extra_contrib_params_count = n_tot
        if initial_values is not None:
            assert len(initial_values) == n_tot
            for i in range(n_tot):
                if initial_values[i] is not None:
                    self.params['ext%02i' % i].value = initial_values[i]
                    self.params['ext%02i' % i].ini_value = initial_values[i]
        # print(self.params)

    def set_spheres(self, spheres):
        """
        Specify the spheres to be fit.

        Paramerer:

            spheres: list of mstm_spectrum.Sphere objects
                If `None` then MSTM will not be run.
        """
        if self.spheres is not None:  # remove parameters of old spheres
            for i in xrange(self.spheres.N):
                self.params.pop('a%02i' % i)
                self.params.pop('x%02i' % i)
                self.params.pop('y%02i' % i)
                self.params.pop('z%02i' % i)
        if spheres is not None:
            self.spheres = spheres
            for i in xrange(self.spheres.N):
                self.params['a%02i' % i] = Parameter('a%02i' % i, self.spheres.a[i])
                self.params['x%02i' % i] = Parameter('x%02i' % i, self.spheres.x[i])
                self.params['y%02i' % i] = Parameter('y%02i' % i, self.spheres.y[i])
                self.params['z%02i' % i] = Parameter('z%02i' % i, self.spheres.z[i])
        else:
            self.set_spheres(ExplicitSpheres())  # empty spheres object

    def _update_spheres(self):
        """
        Set spheres radii and positions to values from params dict
        """
        assert self.spheres is not None
        for i in xrange(len(self.spheres)):
            self.spheres.a[i] = self.params['a%02i' % i].value
            self.spheres.x[i] = self.params['x%02i' % i].value
            self.spheres.y[i] = self.params['y%02i' % i].value
            self.spheres.z[i] = self.params['z%02i' % i].value

    def _update_params(self, values, internal=False):
        """
        Put values from optimized to params

        internal : bool
            if True than internal variables will be updated (scale, bkg, ..)
        """
        try:  # if not iterable (single value in values)
            len(values)
        except:
            print('WARNING: values is not a list')
            values = [values]
        if internal:  # internal (fast) loop parameters
            self.params['scale'].value = values[0]
            for i in range(self.extra_contrib_params_count):
                self.params['ext%02i' % i].value = values[i+1]  # 0th is scale
            print('inner: ', (self.extra_contrib_params_count+1), values)
        else:
            # apply constraints, -- up to now works only for MSTM
            for c in self.constraints:  # apply before
                c.apply(self.params)
            # update params
            i_tot = 0
            for i in range(len(self.spheres)):
                for key in ('a%02i'%i,'x%02i'%i,'y%02i'%i,'z%02i'%i):
                    if self.params[key].varied:
                        self.params[key].value = values[i_tot]
                        i_tot += 1
            assert i_tot == len(values)
            for c in self.constraints:  # and apply after
                c.apply(self.params)
            self.report_result(msg='[%s] Scale: %.3f Bkg: %.2f\n' % (str(datetime.now()),
                self.params['scale'].value, self.params['ext00'].value))  # may be verbous!

    def add_constraint(self, cs):
        """
        Adds constraints on the parameters.
        Usefull for the case of core-shell and layered structures.

        Parameter:

            cs: Contraint object or list of Contraint objects
        """
        try:
            _ = iter(cs)
        except TypeError:
            cs = [cs]
        for c in cs:
            self.constraints.append(c)

    def _get_spectrum(self):
        """
        Calculate the spectrum of agglomerates using mstm_spectrum module.
        """
        if self.stopped():
            raise Exception('Fitting interrupted')

        #~ self._apply_constraints
        if len(self.spheres) > 0:
            spr = SPR(self.wls)
            spr.environment_material = self.MATRIX_MATERIAL

            self._update_spheres()
            spr.set_spheres(self.spheres)
            #~ self.lock.acquire()
            try:
                _, extinction = spr.simulate()
                self.result = np.array(extinction)
            except SpheresOverlapError as e:
                self.chisq = 666  # big evil value
                return np.zeros_like(self.wls)
            except Exception as e:
                print(e)  # let User decide
                raise e
            #~ finally:
                #~ self.lock.release()
        else:  # emty spheres list
            self.result = np.zeros_like(self.wls)

        # perform fast fit over internal variables (scale, bkg, ..)
        values_internal = []
        values_internal.append(self.params['scale'].value)
        for i in range(self.extra_contrib_params_count):
            values_internal.append(self.params['ext%02i' % i].value)

        def _target_func_int(values):
            """ target function for internal fit (fast loop) """
            self._update_params(values, internal=True)

            y_dat = self.exp
            assert self.params['scale'].value == values[0]
            y_fit = values[0] * self.result
            n_tot = 1  # scale is values[0]
            for contribution in self.extra_contributions:
                n = contribution.number_of_params
                y_fit += contribution.calculate(values[n_tot:n_tot+n])
                n_tot += n
            self.chisq = np.sum((y_fit - y_dat)**2)
            #~ self.chisq = np.sum((y_fit - y_dat)**2 * (y_dat/np.max(y_dat)+0.001)) / np.sum((y_dat/np.max(y_dat)+0.001))
            #~ self.chisq = np.sum( (y_fit - y_dat)**2 * y_dat**3 ) * 1E3
            print(self.chisq)
            return self.chisq

        #~ print('/ Internal fit loop /')
        result_int = so.minimize(fun=_target_func_int, x0=values_internal, method='BFGS', tol=self.tolerance,
                                 options={'maxiter':100, 'disp':False})
        values_internal = result_int.x

        self._update_params(values_internal, internal=True)

        self.calc = self.params['scale'].value * self.result
        n_tot = 1  # 0th is scale
        for contribution in self.extra_contributions:
            n = contribution.number_of_params
            self.calc += contribution.calculate(values_internal[n_tot:n_tot+n])
            n_tot += n
        return self.calc

    def get_extra_contributions(self):
        '''
        Return a list of current extra contributions to the spectrum
        '''
        result = []
        values_internal = []
        values_internal.append(self.params['scale'].value)
        for i in range(self.extra_contrib_params_count):
            values_internal.append(self.params['ext%02i' % i].value)
        n_tot = 1  # scale is values[0]
        for contribution in self.extra_contributions:
            n = contribution.number_of_params
            result.append(contribution.calculate(values_internal[n_tot:n_tot+n]))
            n_tot += n
        return result

    def _target_func(self, values):
        """ main target function """
        self._update_params(values)

        y_dat = self.exp
        y_fit = self._get_spectrum()
        self.chisq = np.sum((y_fit - y_dat)**2)
        #~ self.chisq = np.sum((y_fit - y_dat)**2 * (y_dat/np.max(y_dat)+0.001)) / np.sum((y_dat/np.max(y_dat)+0.001))
        #~ self.chisq = np.sum( (y_fit - y_dat)**2 * (y_dat + np.max(y_dat*0.001))**3)
        #~ self.chisq = np.sum((y_fit - y_dat)**2 * y_dat**3) * 1E3
        #print(chisq)
        return self.chisq

    def set_callback(self, func):
        """
        Set callback function which will be called on
        each step of outer optimization loop.

        Parameter:

            func: function(values)
                where values -- list of values passed from optimization routine
        """
        self._cbuser = func

    def _cbplot(self, values):
        """
        callback function
        """
        #~ self.lock.acquire()  # will wait here
        #~ try:
        #print('Scale: %0.3f Bkg: %0.3f ChiSq: %.8f'% (self.params['scale'].value,
        #      self.params['bkg0'].value, self.chisq) )
        if self._cbuser is not None:  # call user-supplied function
            self._cbuser(self, values)
        if self.plot_progress:
            self.line1.set_ydata(self.calc)
            self.fig.canvas.draw()
            #~ plt.pause(0.05)  # this lead of grabbing of the focus by the plot window
            self.fig.canvas.start_event_loop(0.05)
            #from:
            #https://stackoverflow.com/questions/45729092/make-interactive-matplotlib-window-not-pop-to-front-on-each-update-windows-7
            #~ backend = plt.rcParams['backend']
            #~ import matplotlib
            #~ if backend in matplotlib.rcsetup.interactive_bk:
                #~ figManager = matplotlib._pylab_helpers.Gcf.get_active()
                #~ if figManager is not None:
                    #~ canvas = figManager.canvas
                    #~ if canvas.figure.stale:
                        #~ canvas.draw()
                    #~ canvas.start_event_loop(0.05)
        #~ finally:
            #~ self.lock.release()
        #~ input('pe')

    def _apply_constraints(self):
        for c in self.constraints:
            c.apply(self.params)

    def run(self, maxsteps=400):
        """
        Start fitting.

        Parameters:
            maxsteps: int
                limits number of steps performed
        """
        self._apply_constraints()
        # pack parameters to values
        values = []
        for i in range(len(self.spheres)):
            for key in ('a%02i'%i,'x%02i'%i,'y%02i'%i,'z%02i'%i):
                if self.params[key].varied:
                    values.append(self.params[key].value)
        # run optimizer
        result = so.minimize(fun=self._target_func, x0=values, method='Powell', tol=self.tolerance,
                             options={'maxiter':maxsteps, 'disp':True}, callback=self._cbplot)
        self._update_params(result.x)

    def stop(self):
        # https://stackoverflow.com/questions/323972/is-there-any-way-to-kill-a-thread-in-python
        self._stop_event.set()

    def stopped(self):
        return self._stop_event.is_set()

    def report_freedom(self):
        """
        Returns string with short summary before fitting
        """
        self._apply_constraints()
        N = len(self.spheres)
        s = 'Number of spheres:\t%i\n' % N
        n_tot = 0
        for contribution in self.extra_contributions:
            n = contribution.number_of_params
            s += 'Extra contrib. %s with %i params\n' % (contribution.name, n)
            n_tot += n
        s += 'Total number of extra params:\t%i\n' % n_tot
        n_fast = n_slow = n_fix = 0
        for key in self.params:
            if self.params[key].varied:
                if self.params[key].internal_loop:
                    n_fast += 1
                else:
                    n_slow += 1
            else: # not varied
                n_fix += 1
        assert n_fast + n_slow + n_fix == 1 + 4*N + n_tot
        s += 'Degrees of freedom\n'
        s += '\tfast loop:\t%i\n' % n_fast
        s += '\tslow loop:\t%i\n' % n_slow
        print(s)
        return s

    def report_result(self, msg=None):
        """
        Returns string with short summary of fitting results
        """
        s = 'ChiSq:\t%f\n' % self.chisq
        if msg is None:
            s += 'Optimal parameters'
        else:
            s += msg
        for key in sorted(self.params):
            s += '\n\t%s:\t%f\t(Varied:%s)' % (key, self.params[key].value, str(self.params[key].varied))
        print(s)
        return s

if __name__ == '__main__':
    fitter = Fitter('../example/experiment.dat')
    # test Mie fit
    from mstm_studio.contributions import LinearBackground, MieSingleSphere, MieLognormSpheresCached
    from mstm_studio.alloy_AuAg import AlloyAuAg
    fitter.set_extra_contributions([LinearBackground(fitter.wls, 'lin bkg'),
                                    MieLognormSpheresCached(fitter.wls, 'LN Mie')],
                                    [0.02, -0.001,
                                    0.1, 1.5, 0.5])
                                    #~ MieSingleSphere(fitter.wls, 'Mie')],
                                    #~ [0.02, -0.001,
                                    #~ 0.1, 10])
    fitter.extra_contributions[1].set_material(AlloyAuAg(1.), 1.66)
    fitter.extra_contributions[1].plot([0.1, 1.5, 0.5])
    fitter.extra_contributions[1].plot_distrib([0.1, 1.5, 0.5])
    #~ fitter.extra_contributions[1].plot([0.1, 10])
    fitter.set_spheres(None)  # no spheres, no mstm runs
    fitter.report_freedom()
    input('Press enter to run peak fitting')
    fitter.run()
    fitter.report_result()
    contribs = fitter.get_extra_contributions()
    print(contribs)
    input('Press enter to continue')
    # test peak fit
    from contributions import LinearBackground, LorentzPeak
    fitter.set_extra_contributions([LinearBackground(fitter.wls, 'lin bkg'),
                                    LorentzPeak(fitter.wls, 'lorentz peak')],
                                    [0.02, -0.001,
                                     100, 550, 50])
    # fitter.extra_contributions[1].plot([100, 550, 50])
    fitter.set_spheres(None)  # no spheres, no mstm runs
    fitter.report_freedom()
    input('Press enter to run peak fitting')
    fitter.run()
    fitter.report_result()
    input('Press enter to continue')
    # test MSTM fit
    fitter.set_matrix('glass')
    fitter.set_extra_contributions([LinearBackground(fitter.wls, 'lin bkg')], [0.02, -0.001])
    #                         N    X      Y      Z    radius    materials
    spheres = ExplicitSpheres(2, [-1,1], [-2,2], [-3,3], [14,20], [AlloyAuAg(1.), AlloyAuAg(0.)])
    fitter.set_spheres(spheres)
    fitter.add_constraint(ConcentricConstraint(0, 1))
    fitter.add_constraint(FixConstraint('x00', 0))
    fitter.add_constraint(FixConstraint('y00', 0))
    fitter.add_constraint(FixConstraint('z00', 0))
    fitter.add_constraint(RatioConstraint('a00', 'a01', spheres.a[0]/spheres.a[1]))
    fitter.report_freedom()
    input('Press enter to run MSTM fitting')

    fitter.run()
    #~ fitter.start()  # thread method
    #~ fitter.join()   # wait till end
    fitter.report_result()
    #fitter.plot_result()
    #~ y_fit = _get_spectrum( wavelengths, values )
    #~ plt.plot( wavelengths, exp, wavelengths, y_fit )
    #~ #plt.axis([0, 1, 1.1*np.amin(s), 2*np.amax(s)])
    #~ plt.xlabel('Wavelength, nm')
    #~ plt.ylabel('Exctinction, a.u.')
    #~ plt.show()
    input('Press enter to finish')
    print('It is over.')



"""
  code from nanophotonics/npmie repository
  url: https://github.com/nanophotonics/npmie
  code by Alan Sanders

  modified to use in MSTM-studio by L.Aavakyan
"""

import numpy as np
from mstm_studio.mstm_spectrum import Material

try:
    from scipy.special import sph_jnyn
except:
    from scipy.special import spherical_yn, spherical_jn
    def sph_jnyn(maxn, z):
        jn  = []
        djn = []
        yn  = []
        dyn = []
        for n in range(0, maxn+1):
            jn.append (spherical_jn(n, z))
            djn.append(spherical_jn(n, z, derivative=True))
            yn.append (spherical_yn(n, z))
            dyn.append(spherical_yn(n, z, derivative=True))
        return np.array(jn), np.array(djn), np.array(yn), np.array(dyn)


def sph_hn(n, x):
    # calculate spherical hankel, h(n,x) = j(n,x) + iy(n,x) #
    jn, djn, yn, dyn = sph_jnyn(n, x)
    hn = jn + 1j * yn
    dhn = djn + 1j * dyn
    return hn, dhn


def calculate_mie_coefficients(n_max, x, m):
    """
    Calculates the Mie coefficients.

    :rtype : object
    :param n_max:
    :param x: size parameter
    :param m:
    """

    # calculate spherical bessels #
    jn, djn, yn, dyn = sph_jnyn(n_max, x)      # j(n, x), y(n, x)
    jm, djm, ym, dym = sph_jnyn(n_max, m * x)  # j(n, mx), y(n, mx)
    # calculate spherical hankel #
    hn, dhn = sph_hn(n_max, x)                 # h(n, x)
    # calculate riccati bessel functions #
    dpsi_n = [x * jn[n-1] - n * jn[n] for n in range(0, len(jn))]
    dpsi_m = [m * x * jm[n-1] - n * jm[n] for n in range(0, len(jm))]
    dzeta_n = [x * hn[n-1] - n * hn[n] for n in range(0, len(hn))]

    a_n = (m**2 * jm * dpsi_n - jn * dpsi_m) / (m**2 * jm * dzeta_n - hn * dpsi_m)
    b_n = (jm * dpsi_n - jn * dpsi_m) / (jm * dzeta_n - hn * dpsi_m)
    return a_n, b_n


def calculate_mie_efficiencies(r, wavelength, n_sph, n_med):
    """
    Calculates the mie efficiencies (q_scat, q_abs, q_ext, q_bscat)
    for a sphere in a dielectric medium at a given wavelength.

    :rtype : object
    :param r: radius of the sphere
    :param wavelength: wavelength of illumination
    :param n_sph: complex refractive index of the sphere
    :param n_med: real refractive index of the dielectric medium
    :return:
    """

    # calculate size parameter #
    x = n_med * (2 * np.pi / wavelength) * r    # x = n_med * kr, size parameter
    m = n_sph / n_med
    # n_max = int(np.ceil(x.real)+1)      # number of terms in series expansion
    n_max = int(x + 4 * x**(1.0 / 3.0) + 2)  # number of terms in series expansion

    q_scat = 0
    q_bscat = 0
    q_ext = 0
    q_abs = 0
    a_n, b_n = calculate_mie_coefficients(n_max, x, m)
    a = 0
    b = 0
    for n in range(1, n_max):
        a += a_n[n]
        b += b_n[n]
        q_scat += (2 * n + 1) * (abs(a_n[n])**2 + abs(b_n[n])**2)
        q_bscat += (2 * n + 1) * ((-1)**n) * (abs(a_n[n])**2 + abs(b_n[n])**2)
        q_ext += (2 * n + 1) * (a_n[n] + b_n[n]).real
    q_scat *= 2 / x**2
    q_bscat *= 2 / x**2
    q_ext *= 2 / x**2
    q_abs = q_ext - q_scat
    return q_scat, q_bscat, q_ext, q_abs


def calculate_mie_spectra(wavelengths, r, material, n_medium=1.):
    """
    Calculates the mie scattering and extinction efficiency of spherical
    nanoparticles with radius r and given material surrounded by a medium n_med
    for a set of given wavelengths.
    :rtype : object
    :param wavelengths: array of wavelengths to calculate spectra from
    :param r: radius of the sphere
    :param material: instance of Material class
    :param n_med: refractive index of the surrounding dielectric medium
    """

    mie_scattering = []
    mie_backscattering = []
    mie_extinction = []
    mie_absorption = []
    for wl in wavelengths:
        n_sph = material.get_n(wl) + 1j * material.get_k(wl)
        q_scat, q_bscat, q_ext, q_abs = calculate_mie_efficiencies(
            r, wl, n_sph, n_medium
        )
        mie_scattering.append(q_scat)
        mie_backscattering.append(q_bscat)
        mie_extinction.append(q_ext)
        mie_absorption.append(q_abs)
    return (np.array(mie_scattering), np.array(mie_backscattering),
            np.array(mie_extinction), np.array(mie_absorption))


if __name__ == '__main__':
    import matplotlib.pyplot as plt
    #~ diameter_np = raw_input('Enter nanoparticle diameter (nm): ')
    #~ material = raw_input("Enter nanoparticle material: ")
    #~ medium = raw_input("Enter surrounding medium: ")
    diameter_np = material = medium = ''  # test
    if diameter_np == '':
        diameter_np = 140.
    else:
        diameter_np = float(diameter_np)
    if material == '':
        material = 'Au'
    if medium == '':
        medium = 1.
    else:
        medium = float(medium)
    mat_dict = {'Au': 'etaGold.txt', 'Ag': 'etaSilver.txt'}
    material_object = Material(3) # Material(mat_dict[material])
    wavelength = np.arange(300, 1000, 0.1)
    mie_scattering, mie_backscattering, mie_extinction, \
        mie_absorption = calculate_mie_spectra(
            wavelength, diameter_np / 2.0, material_object, medium
        )
    # save to file
    data = np.stack([wavelength, mie_scattering, mie_backscattering, \
        mie_extinction, mie_absorption])
    np.savetxt('MIE.dat', np.transpose(data), header='wl\tscatt\tbscatt\text\tabs')
    fig = plt.figure()
    # wavelength plots #
    ax = fig.add_subplot(411)
    ax.plot(wavelength, mie_scattering, 'r', label='scattering')
    ax.set_xticklabels(ax.get_xticklabels(), visible=False)
    ax.set_ylabel('scattering')
    ax = fig.add_subplot(412)
    ax.plot(wavelength, mie_backscattering, 'k', label='back-scattering')
    ax.set_xticklabels(ax.get_xticklabels(), visible=False)
    ax.set_ylabel('back-scattering')
    ax = fig.add_subplot(413)
    ax.plot(wavelength, mie_extinction, 'b', label='extinction')
    ax.set_xticklabels(ax.get_xticklabels(), visible=False)
    ax.set_ylabel('extinction')
    ax = fig.add_subplot(414)
    ax.plot(wavelength, mie_absorption, 'g', label='absorption')
    ax.set_ylabel('absorption')
    ax.set_xlabel('wavelength (nm)')
    plt.tight_layout()
    plt.show()



# -*- coding: utf-8 -*-
#
# ----------------------------------------------------- #
#                                                       #
#  This code is a part of T-matrix fitting project      #
#  Contributors:                                        #
#   L. Avakyan <laavakyan@sfedu.ru>                     #
#   K. Yablunovskiy <kirill-yablunovskii@mail.ru>       #
#                                                       #
# ----------------------------------------------------- #
"""
Contributions to UV/vis extinction spectra other
then obtained from MSTM.
"""
from __future__ import print_function
from __future__ import division
import numpy as np
try:
    import matplotlib.pyplot as plt
except:
    pass

# use input in both python2 and python3
try:
   input = raw_input
except NameError:
   pass
# use xrange in both python2 and python3
try:
    xrange
except NameError:
    xrange = range

try:
    from film_exctinction import gold_film_ex  # for gold film background
except:
    pass

try:
    from mstm_studio.mie_theory import calculate_mie_spectra
except:
    pass


class Contribution(object):
    """
    Abstract class to include contributions other then
    calculated by MSTM. All lightweight calculated
    contribtions (constant background, lorentz and guass peaks, Mie, etc.)
    should enhirit from it.
    """
    number_of_params = 0  # Should be another value in child class

    def __init__(self, wavelengths=[], name='ExtraContrib'):
        """
        Parameters:

            wavelengths: list or numpy array
                wavelengths in nm

            name: string
                optional label

        """
        self.name = name
        self.set_wavelengths(wavelengths)

    def set_wavelengths(self, wavelengths):
        """
        Modify wavelengths
        """
        self.wavelengths = np.array(wavelengths)

    def calculate(self, values):
        """
        This method should be overriden in child classes.

        Parameters:

            values: list of control parameters

        Return:

            numpy array of contribution values at specified wavelengths
        """
        self._check(values)
        return np.zeros(len(self.wavelengths))

    def _check(self, values):
        if len(values) < self.number_of_params:
            raise Exception('Too few values! '+str(values))

    def plot(self, values, fig=None, axs=None):
        """
        plot contribution

        Parameters:

            values: list of parameters

            fig: matplotlib figure

            axs: matplotlib axes

        Return:

            filled/created fig and axs objects
        """
        flag = fig is None
        if flag:
            fig = plt.figure()
            axs = fig.add_subplot(111)
        x = self.wavelengths
        y = self.calculate(values)
        axs.plot(x, y, 'g--', label=self.name)
        axs.set_ylabel('Intensity')
        axs.set_xlabel('Wavelength, nm')
        axs.legend()
        if flag:
            plt.show()
        return fig, axs


class ConstantBackground(Contribution):
    """
    Constant background contribution :math:`f(\lambda) = bkg`.
    """
    number_of_params = 1

    def calculate(self, values):
        """
        Parameters:

            values: [bkg]

        Return:

            numpy array
        """
        self._check(values)
        return values[0] * np.ones(len(self.wavelengths))


class LinearBackground(Contribution):
    """
    Two-parameter background :math:`f(\lambda) = a \cdot \lambda + b`.
    """
    number_of_params = 2

    def calculate(self, values):
        """
        Parameters:

            values: list of control parameters `scale`, `mu` and `Gamma`

        Return:

            numpy array
        """
        self._check(values)
        return values[0] + values[1] * self.wavelengths


class LorentzPeak(Contribution):
    """
    Lorentz function

    .. math::

        L(\lambda) = \\frac {scale} {(\lambda-\mu)^2 + \Gamma^2}

    """
    number_of_params = 3

    def calculate(self, values):
        """
        Parameters:

            values: list of control parameters `scale`, `mu` and `Gamma`

        Return:

            numpy array
        """
        self._check(values)
        return values[0] / ((self.wavelengths - values[1])**2 + (values[2])**2)


class GaussPeak(Contribution):
    """
    Gauss function

    .. math::

        G(\lambda) = scale \cdot \exp\left( - \\frac{(\lambda-\mu)^2}{2\sigma^2} \\right)

    """
    number_of_params = 3

    def calculate(self, values):
        """
        Parameters:

            values: list of control parameters `scale`, `mu` and `sigma`

        Return:

            numpy array
        """
        self._check(values)
        return values[0] * np.exp(-(self.wavelengths - values[1])**2 / (2 * values[2]**2))


class LorentzBackground(Contribution):
    """
    Lorentz peak in background. Peak center is fixed.

    .. math::

        L(\lambda) = \\frac {scale} {(\lambda-center)^2 + \Gamma^2}

    """
    number_of_params = 2
    center = 250

    def calculate(self, values):
        self._check(values)
        return values[0] / ((self.wavelengths-self.center)**2 + (values[1])**2)


class FilmBackground(Contribution):
    """
    Background interpolated from experimental spectra of gold foil
    """
    number_of_params = 3

    def calculate(self, values):
        """ TODO """
        self._check(values)
        return values[0] + values[1] * gold_film_ex(values[2], self.wavelengths)


class MieSingleSphere(Contribution):
    """
    Mie contribution from single sphere.

    Details are widely discusses, see, for example [Kreibig_book1995]_
    """
    number_of_params = 2
    material = None  # instance of mstm_spectrum.Material
    matrix = 1.0     # medium refractive index

    def calculate(self, values):
        """
        Parameters:

            values: list of control parameters `scale`, `diameter`

        Return:

            extinction efficiency array of Mie sphere
        """
        self._check(values)
        if self.material is None:
            raise Exception('Mie calculation requires material data. Stop.')
        D = np.abs(values[1])
        self.material.D = D
        _, _, mie_extinction, _ = calculate_mie_spectra(
            self.wavelengths, D, self.material, self.matrix)
        return values[0] * mie_extinction

    def set_material(self, material, matrix=1.0):
        """
        Define the material of sphere and environment

        Parameters:

            material: Material object
                material of the sphere

            matrix: float, string or Material object
                material of the environment

        Return:

            True if properties were changed, False - otherwise.

        """
        changed = False
        try:
            matr = float(matrix)
        except:
            matr = matrix.get_n(550)  # assume it is Material instance
        if matr != self.matrix:
            self.matrix = matr
            changed = True
        try:
            material.get_n(550)
            material.get_k(550)
        except:
            raise Exception('Bad material object')
        if material is not self.material:
            self.material = material
            changed = True
        return changed


class MieLognormSpheres(MieSingleSphere):
    """
    Mie contribution from an ensemble of spheres
    with sizes distributed by Log-Normal law
    """
    number_of_params = 3
    diameters = np.logspace(0, 3, 301)
    MAX_DIAMETER_TO_PLOT = 100

    def lognorm(self, x, mu, sigma):
        """
        The shape of Log-Normal distribution:

        .. math::

            LN(D) = \\frac {1}{D \sigma \sqrt{2\pi}} \exp\left( - \\frac{(\log(D)-\mu)^2}{2\sigma^2} \\right)
        """
        return (1.0/(x*sigma*np.sqrt(2*np.pi)))*np.exp(-((np.log(x)-mu)**2)/(2*sigma**2))

    def calculate(self, values):
        """
        Parameters:

            values: list of control parameters `scale`, `mu` and `sigma`

        Return:

            Mie extinction efficiency of log-normally distributed spheres
        """
        self._check(values)
        dD = np.ediff1d(self.diameters, to_begin=1e-3)
        distrib = self.lognorm(self.diameters, np.abs(values[1]), np.abs(values[2]))
        result = np.zeros_like(self.wavelengths)
        for diameter, count in zip(self.diameters, distrib*dD):
            self.number_of_params = 2  # else will get error on a check
            mie_ext = super(MieLognormSpheres, self).calculate(values=[1.0, diameter/2.0])
            self.number_of_params = 3  # ugly, but everything has a price
            result += count * mie_ext
        return values[0] * result

    def plot_distrib(self, values, fig=None, axs=None):
        """
        Plot size distribution

        Parameters:

            values: list of control parameters

            fig: matplotlib figure

            axs: matplotlib axes

        Return:

            filled/created fig and axs objects
        """
        flag = fig is None
        if flag:
            fig = plt.figure()
            axs = fig.add_subplot(111)
        x = self.diameters[self.diameters < self.MAX_DIAMETER_TO_PLOT]
        y = self.lognorm(x, np.abs(values[1]), np.abs(values[2]))
        axs.plot(x, y, 'b', label=self.name)
        axs.set_ylabel('Count')
        axs.set_xlabel('Diameter, nm')
        axs.legend()
        if flag:
            plt.show()
        return fig, axs

    def set_material(self, material, matrix=1.0):
        print('matrix: %s' % matrix)
        if super().set_material(material, matrix):
            self._M = None  # clear cache if changed materials


class MieLognormSpheresCached(MieLognormSpheres):
    """
    Mie contribution from an ensemble of spheres
    with sizes distributed by Lognormal law.

    Cached version - use it to speed-up fitting.
    """
    number_of_params = 3
    diameters = np.logspace(0, 3, 301)
    MAX_DIAMETER_TO_PLOT = 100
    _M = None  # cache matrix, None after initialization

    def lognorm(self, x, mu, sigma):
        return (1.0/(x*sigma*np.sqrt(2*np.pi)))*np.exp(-((np.log(x)-mu)**2)/(2*sigma**2))

    def calculate(self, values):
        """
        Parameters:

            values: list of control parameters `scale`, `mu` and `sigma`

        Return:

            Mie extinction efficiency of log-normally distributed spheres
        """
        self._check(values)
        if self._M is None:  # initialize cache matrix
            print('Building cache...')
            self._M = np.zeros(shape=(len(self.wavelengths), len(self.diameters)))
            self.number_of_params = 2  # else will get error on a check
            for i, diameter in enumerate(self.diameters):
                self._M[:, i] = super(MieLognormSpheres, self).calculate(values=[1.0, diameter/2.0])
            self.number_of_params = 3  # ugly, but everything has a price
            print('Building cache... done')

        dD = np.ediff1d(self.diameters, to_begin=1e-3)
        distrib = self.lognorm(self.diameters, np.abs(values[1]), np.abs(values[2]))
        result =  np.dot(self._M, distrib * self.diameters**2 * dD) / np.sum(distrib * self.diameters**2 * dD)
        return values[0] * result


if __name__=='__main__':
    # tests come here
    # ~ cb = ConstantBackground(name='const', wavelengths=[300,400,500,600,700,800])
    # ~ print(cb.calculate([3]))
    # ~ cb.plot([3])
    # ~ mie = MieSingleSphere(name='mie', wavelengths=np.linspace(300,800,50))
    # ~ mie = MieLognormSpheres(name='mie', wavelengths=np.linspace(300,800,50))
    from mstm_studio.contributions import MieLognormSpheresCached
    from mstm_studio.alloy_AuAg import AlloyAuAg
    mie = MieLognormSpheresCached(name='mie', wavelengths=np.linspace(300,800,50))
    mie.set_material(AlloyAuAg(x_Au=1), 1.66)
    mie.plot([1,1.5,0.5])  # scale mu sigma
    print('See you!')





from mstm_studio.mstm_spectrum import SPR, Material, SpheresOverlapError
import subprocess
import os   # to delete files after calc.
import sys  # to check whether running on Linux or Windows
import tempfile
import datetime
import numpy as np
try:
    import matplotlib.pyplot as plt
except ImportError:
    pass


class NearField(SPR):
    '''
    Calculate field distribution map at fixed wavelength
    '''

    def __init__(self, wavelength):
        super().__init__([wavelength])
        self.set_incident_field(fixed=True, azimuth_angle=0.0,
                                polar_angle=0.0, polarization_angle=0.0)
        self.set_plane()
        self.paramDict['calculate_near_field'] = 1  # do it

    def set_plane(self, plane='zx', hmin=-10., hmax=10.,
                  vmin=-10., vmax=10., step=1., offset=0.):
        '''
        Determine the plane and grid for near field computation.

        plane: one of 'yz'|'zx'|'xy'
        hmin, hmax, vmin, vmax: horizontal and vertical sizes
        step:   size of the grid grain
        offset: shift of the plane
        '''
        if plane == 'zy':
            plane = 'yz'
        if plane == 'xz':
            plane = 'zx'
        if plane == 'yx':
            plane = 'xy'
        # 1: y - z plane;  2: z - x plane;  3: x - y
        if plane == 'yz':
            self.paramDict['near_field_plane_coord'] = 1
        elif plane == 'zx':
            self.paramDict['near_field_plane_coord'] = 2
        elif plane == 'xy':
            self.paramDict['near_field_plane_coord'] = 3
        else:
            raise Exception('Wrong plane specification! \n %s' % plane)
        # TODO: extend to calculate complex E vector = 1,
        #       complex E and H vectors = 2
        self.paramDict['near_field_output_data'] = 0  # |E|^2
        self.paramDict['near_field_output_file'] = 'nf-temp.dat'

        self.hmin = hmin
        self.hmax = hmax
        self.vmin = vmin
        self.vmax = vmax
        self.step = step
        self.offset = offset
        self.nh = int(np.round((hmax - hmin) / step)) + 1
        self.nv = int(np.round((vmax - vmin) / step)) + 1
        print('Field computation grid: %ix%i' % (self.nh, self.nv))

    def simulate(self):
        '''
        Run MSTM code to produce 2D map of field distribution.
        '''
        if self.paramDict['number_spheres'] == 0:  # np spheres
            return self.wavelengths, np.zeros_like(self.wavelengths)
        if self.spheres.check_overlap():
            raise SpheresOverlapError('Spheres overlapping!')
        if isinstance(self.environment_material, Material):
            material = self.environment_material
        else:
            print(self.environment_material)
            material = Material(self.environment_material)
        with tempfile.TemporaryDirectory() as tmpdir:  # COPIED FROM PARENT!
            print('Using temporary directory: %s' % tmpdir)
            outFID = open(os.path.join(tmpdir, 'scriptParams.inp'), 'w')
            outFID.write('begin_comment\n')
            outFID.write('**********************************\n')
            outFID.write('  MSTM input for NearField calculation\n')
            outFID.write('  Generated by python script\n')
            outFID.write('  %s\n' %
                         datetime.datetime.now().strftime('%Y-%m-%d %H:%M'))
            outFID.write('**********************************\n')
            outFID.write('end_comment\n')
            for key in self.paramDict.keys():
                if key not in self.local_keys:
                    outFID.write(key + '\n')
                    if isinstance(self.paramDict[key], str):
                        svalue = self.paramDict[key]
                    else:
                        if isinstance(self.paramDict[key], list):
                            svalue = '  '.join(map(str, self.paramDict[key]))
                        else:
                            svalue = str(self.paramDict[key])
                        # replace exponent symbol
                        svalue = svalue.replace('e', 'd', 1)
                    outFID.write('%s \n' % svalue)

            lam = self.wavelengths[0]
            k = 2.0 * 3.14159 / lam
            outFID.write('begin_comment\n')
            outFID.write('**********************************\n')
            outFID.write('  Wavelength  %.3f \n' % lam)
            outFID.write('**********************************\n')
            outFID.write('end_comment\n')
            outFID.write('output_file\n')
            outFID.write('mstm_l%.0f.out\n' % (lam * 1000))
            outFID.write('length_scale_factor\n')
            outFID.write('  %.6f\n' % k)
            outFID.write('medium_real_ref_index\n')
            outFID.write('  %f\n' % material.get_n(lam))
            outFID.write('medium_imag_ref_index\n')
            outFID.write('  %f\n' % material.get_k(lam))

            outFID.write('spacial_step_size\n')
            outFID.write('  %f\n' % (k * self.step))
            outFID.write('near_field_plane_position\n')
            outFID.write('  %f\n' % (k * self.offset))
            outFID.write('near_field_plane_vertices\n')
            outFID.write('  %f  %f  %f  %f\n' % (k * self.hmin, k * self.vmin,
                                                 k * self.hmax, k * self.vmax))

            outFID.write('sphere_sizes_and_positions\n')

            for i in range(len(self.spheres)):
                a = self.spheres.a[i]
                if a > 0:  # consider only positive radii
                    x = self.spheres.x[i]
                    y = self.spheres.y[i]
                    z = self.spheres.z[i]
                    n = self.spheres.materials[i].get_n(lam)
                    k = self.spheres.materials[i].get_k(lam)
                    outFID.write('  %.4f  %.4f  %.4f  %.4f  %.3f  %.3f \n' %
                                 (a, x, y, z, n, k))

            outFID.write('end_of_options\n')
            outFID.close()

            # run the binary
            if sys.platform == 'win32':
                si = subprocess.STARTUPINFO()
                si.dwFlags |= subprocess.STARTF_USESHOWWINDOW
                subprocess.call('%s scriptParams.inp > NUL' % self.command,
                                shell=True, startupinfo=si, cwd=tmpdir)
            else:
                subprocess.call('%s scriptParams.inp > /dev/null' %
                                self.command, shell=True, cwd=tmpdir)

            # parse the simulation results
            fn = os.path.join(tmpdir,
                              self.paramDict['near_field_output_file'])
            with open(fn) as fout:
                fout.readline()  # skip 1st
                nsph = int(fout.readline().strip())  # no. of spheres in plane
            data = np.loadtxt(fn, skiprows=2 + nsph)
            self.field = np.reshape(data[:, 2], [self.nh, self.nv])
            return self.field

    def _get_grid_hv(self):
        h = np.arange(self.hmin, self.hmax + self.step / 2., self.step)
        v = np.arange(self.vmin, self.vmax + self.step / 2., self.step)
        return h, v

    def write(self, filename):
        ''' save field data to text file'''
        h, v = self._get_grid_hv()
        with open(filename, 'w') as fout:
            if self.paramDict['near_field_plane_coord'] == 1:
                fout.write('# Y[nm]\tZ[nm]\t|E|^2\r\n')
            elif self.paramDict['near_field_plane_coord'] == 2:
                fout.write('# Z[nm]\tX[nm]\t|E|^2\r\n')
            elif self.paramDict['near_field_plane_coord'] == 3:
                fout.write('# X[nm]\tY[nm]\t|E|^2\r\n')

            for i, x in enumerate(h):
                for j, y in enumerate(v):
                    fout.write('%.4f\t%.4f\t%.8f\r\n' % (x, y,
                                                         self.field[i, j]))

    def plot(self):
        '''
        Show 2D field distribution
        '''
        x, y = self._get_grid_hv()
        xx, yy = np.meshgrid(x, y)
        xx = np.transpose(xx)
        yy = np.transpose(yy)
        zz = self.field
        plt.pcolormesh(xx, yy, zz, cmap='hot', shading='auto')
        plt.colorbar()
        if self.paramDict['near_field_plane_coord'] == 1:
            plt.xlabel('Y, nm')
            plt.ylabel('Z, nm')
        elif self.paramDict['near_field_plane_coord'] == 2:
            plt.xlabel('Z, nm')
            plt.ylabel('X, nm')
        elif self.paramDict['near_field_plane_coord'] == 3:
            plt.xlabel('X, nm')
            plt.ylabel('Y, nm')
        plt.gca().set_aspect('equal')
        plt.show()
        return plt


if __name__ == '__main__':
    from mstm_studio.mstm_spectrum import Material, ExplicitSpheres
    mat1 = Material(os.path.join('nk', 'etaSilver.txt'))
    nf = NearField(wavelength=340)
    nf.environment_material = 'glass'
    nf.set_plane(plane='xz', hmin=-10, hmax=20, vmin=-15, vmax=15, step=0.25)

    spheres = ExplicitSpheres(2, [0, 0, 0, 5, 0, 0, 11, 3],
                              mat_filename=2*[mat1])
    nf.set_spheres(spheres)
    nf.simulate()
    nf.plot()
    nf.write('nearfield.dat')

    print('See you!')



#! /usr/bin/env python
#
# GUI module was first generated by PAGE version 4.9
# In conjunction with Tcl version 8.6
#    Dec 04, 2017 01:09:15 AM
# manually edited afterwards
import sys
import os
try:
    from Tkinter import Tk, Toplevel, Canvas, Menu, Pack, Grid, Place
except ImportError:
    from tkinter import Tk, Toplevel, Canvas, Menu, Pack, Grid, Place

try:
    import ttk
    py3 = False
except ImportError:
    import tkinter.ttk as ttk
    py3 = True

from PIL import ImageTk

import mstm_studio.mstm_studio_support as sup

import time  # to test splash

def vp_start_gui():
    '''Starting point when module is the main routine.'''
    global val, w, root
    root = Tk()
    top = MSTM_studio (root)
    sup.init(root, top)
    root.mainloop()

w = None
def create_MSTM_studio(root, *args, **kwargs):
    '''Starting point when module is imported by another program.'''
    global w, w_win, rt
    rt = root
    w = Toplevel (root)
    top = MSTM_studio (w)
    sup.init(w, top, *args, **kwargs)
    return (w, top)

def destroy_MSTM_studio():
    global w
    w.destroy()
    w = None


class MSTM_studio:

    splash_time = 0.5  # time to show splash window, seconds

    def __init__(self, top=None):
        '''This class configures and populates the toplevel window.
           top is the toplevel containing window.'''
        top.withdraw()
        time_start = time.time()
        splash = sup.SplashWindow(top)
        self.style = ttk.Style()
        if sys.platform == 'win32':
            self.style.theme_use('winnative')
        self.style.configure('.',font='TkDefaultFont')

        #~ top.geometry('838x455+364+117')
        top.geometry('850x440')
        top.title('MSTM studio')
        #~ top.configure(highlightcolor='black')
        self.load_images()
        self.root_panedwin = ttk.Panedwindow(top, orient='horizontal')
        self.root_panedwin.place(relx=0.0, rely=0.0, relheight=1.0, relwidth=1.0)

        self.root_panedwin.configure(width=200)
        self.left_frame = ttk.Frame(width=220.0)
        self.root_panedwin.add(self.left_frame)
        #~ self.middle_frame = ttk.Labelframe(width=350, text='View')
        self.middle_frame = ttk.Frame(width=350)
        self.root_panedwin.add(self.middle_frame)
        self.right_frame = ttk.Frame()
        self.root_panedwin.add(self.right_frame)
        self.__funcid0 = self.root_panedwin.bind('<Map>', self.__adjust_sash0)

        self.left_panedwin = ttk.Panedwindow(self.left_frame, orient='vertical')
        self.left_panedwin.place(relx=0.0, rely=0.0, relheight=1.0, relwidth=1.0)
        self.left_panedwin.configure(width=200)

        self.materials_frame = ttk.Labelframe(height=105, text='Materials')
        self.left_panedwin.add(self.materials_frame)
        self.spheres_frame = ttk.Labelframe(text='Spheres')
        self.left_panedwin.add(self.spheres_frame)
        self.__funcid1 = self.left_panedwin.bind('<Map>', self.__adjust_sash1)

        self.style.configure('Treeview.Heading',  font='TkDefaultFont')
        self.stvMaterial = ScrolledTreeView(self.materials_frame)
        self.stvMaterial.place(relx=0.0, y=30, relheight=0.8, relwidth=1.0)
        self.configure_stvMaterial()
        self.stvMaterial.bind('<Double-1>', sup.btChangeMatColClick)

        self.btAddMat = ttk.Button(self.materials_frame, command=sup.btAddMatClick,
                                   text='A', image=self.imAdd)
        self.btAddMat.place(x=5, y=0, height=25, width=25)

        self.btLoadMat = ttk.Button(self.materials_frame, command=sup.btLoadMatClick,
                                    text='L', image=self.imLoad)
        self.btLoadMat.place(x=30, y=0, height=25, width=25)

        self.btPlotMat = ttk.Button(self.materials_frame, command=sup.btPlotMatClick,
                                    text='P', image=self.imPlot)
        self.btPlotMat.place(x=55, y=0, height=25, width=25)

        self.btDelMat = ttk.Button(self.materials_frame, command=sup.btDelMatClick,
                                   text='D', image=self.imDelete)
        self.btDelMat.place(relx=1, x=-30, rely=0, height=25, width=25)

        self.stvSpheres = ScrolledTreeView(self.spheres_frame)
        self.stvSpheres.place(relx=0.0, y=30, relheight=0.85, relwidth=1.0)
        self.configure_stvSpheres()
        self.stvSpheres.bind('<Double-1>', sup.btEditSphClick)

        self.btAddSph = ttk.Button(self.spheres_frame, command=sup.btAddSphClick,
                                   text='A', image=self.imAdd)
        self.btAddSph.place(x=5, y=0, height=25, width=25)

        self.btEditSph = ttk.Button(self.spheres_frame, command=sup.btEditSphClick,
                                    text='E', image=self.imEdit)
        self.btEditSph.place(x=30, y=0, height=25, width=25)

        self.btPlotSph = ttk.Button(self.spheres_frame, command=sup.btPlotSphClick,
                                    text='R', image=self.imRefresh)
        self.btPlotSph.place(x=55, y=0, height=25, width=25)

        self.lbEnvMat = ttk.Label(self.spheres_frame, text='Matrix')
        self.lbEnvMat.place(relx=0.45, y=-5)
        self.cbEnvMat = ttk.Combobox(self.spheres_frame)
        self.cbEnvMat.place(relx=0.45, y=10, width=55)

        self.btDelSph = ttk.Button(self.spheres_frame, command=sup.btDelSphClick,
                                   text='D', image=self.imDelete)
        self.btDelSph.place(relx=1.0, y=0, x=-30, height=25, width=25)

        self.middle_panedwin = ttk.Panedwindow(self.middle_frame, orient='vertical')
        self.middle_panedwin.place(relx=0.0, rely=0.0, relheight=1.0, relwidth=1.0)
        #~ self.middle_panedwin.configure(relwidth=1.0)
        self.canvas_frame = ttk.Labelframe(height=360, text='View')
        self.middle_panedwin.add(self.canvas_frame)
        self.spectrum_frame = ttk.Labelframe(height=-40, text='Spectrum')
        self.middle_panedwin.add(self.spectrum_frame)
        self.__funcid2 = self.left_panedwin.bind('<Map>', self.__adjust_sash2)

        self.canvas = Canvas(self.canvas_frame)
        self.canvas.place(relx=0.0, rely=0, relheight=1.0, relwidth=1.0)
        self.canvas.configure(background='white')
        self.canvas.configure(borderwidth='2')
        self.canvas.configure(relief='ridge')
        self.canvas.configure(selectbackground='#c4c4c4')
        self.canvas.bind('<Button-4>',   sup.mouse_wheel)  # for Linux
        self.canvas.bind('<Button-5>',   sup.mouse_wheel)  # for Linux
        self.canvas.bind('<MouseWheel>', sup.mouse_wheel)  # for Windowz
        self.canvas.bind('<Button-3>',   sup.mouse_down)
        self.canvas.bind('<B3-Motion>',  sup.mouse_move)
        self.canvas.bind('<ButtonRelease-3>',  sup.mouse_up)

        self.lbZoom = ttk.Label(self.canvas, text='x1.00', background='white')  #font=('courier', 18, 'bold'), width=10)
        self.lbZoom.place(relx=1.0, x=-50, rely=1.0, y=-25)

        self.lbLambdaMin = ttk.Label(self.spectrum_frame, text='min')
        self.lbLambdaMin.place(x=5, y=0)
        self.edLambdaMin = ttk.Entry(self.spectrum_frame)
        self.edLambdaMin.place(x=5, y=15, width=35)
        self.edLambdaMin.insert(0, '300')

        self.lbLambdaMax = ttk.Label(self.spectrum_frame, text='max')
        self.lbLambdaMax.place(x=45, y=0)
        self.edLambdaMax = ttk.Entry(self.spectrum_frame)
        self.edLambdaMax.place(x=45, y=15, width=35)
        self.edLambdaMax.insert(0, '800')

        self.lbLambdaCount = ttk.Label(self.spectrum_frame, text='count')
        self.lbLambdaCount.place(x=85, y=0)
        self.edLambdaCount = ttk.Entry(self.spectrum_frame)
        self.edLambdaCount.place(x=85, y=15, width=35)
        self.edLambdaCount.insert(0, '51')

        self.btCalcSpec = ttk.Button(self.spectrum_frame, command=sup.btCalcSpecClick,
                                     text='Calculate', image=self.imCalc, compound='left')
        self.btCalcSpec.place(x=130, y=10, width=90, height=25)

        self.lbSpecScale = ttk.Label(self.spectrum_frame, text='scale')
        self.lbSpecScale.place(relx=1, x=-115, y=0)
        self.edSpecScale = ttk.Entry(self.spectrum_frame)
        self.edSpecScale.place(relx=1, x=-115, y=15, width=50)
        self.edSpecScale.insert(0, '1')

        self.btSaveSpec = ttk.Button(self.spectrum_frame, command=sup.btSaveSpecClick,
                                     text='S', image=self.imSave)
        self.btSaveSpec.place(relx=1, x=-55, y=10, width=25, height=25)

        self.btPlotSpec = ttk.Button(self.spectrum_frame, command=sup.btPlotSpecClick,
                                     text='P', image=self.imPlot)
        self.btPlotSpec.place(relx=1, x=-30, y=10, width=25, height=25)

        self.right_panedwin = ttk.Panedwindow(self.right_frame, orient='vertical')
        self.right_panedwin.place(relx=0.0, rely=0.0, relheight=1.0, relwidth=1.0)

        self.right_panedwin.configure(width=200)
        self.plot_frame = ttk.Labelframe(height=200, text='Plot')
        self.right_panedwin.add(self.plot_frame)
        self.contribs_frame = ttk.Labelframe(height=150, text='Other contributions')
        self.right_panedwin.add(self.contribs_frame)
        self.fitting_frame = ttk.Labelframe(height=-50, text='Fitting')
        self.right_panedwin.add(self.fitting_frame)
        self.__funcid3 = self.right_panedwin.bind('<Map>', self.__adjust_sash3)

        # CONTRIBUTIONS
        self.btAddContrib = ttk.Button(self.contribs_frame, command=sup.btAddContribClick,
                                   text='A', image=self.imAdd)
        self.btAddContrib.place(x=5, y=0, height=25, width=25)

        self.btPlotAllContribs = ttk.Button(self.contribs_frame, command=sup.btPlotAllContribsClick,
                                            text='P', image=self.imPlot)
        self.btPlotAllContribs.place(x=30, y=0, height=25, width=25)

        self.btDelContrib = ttk.Button(self.contribs_frame, command=sup.btDelContribClick,
                                   text='D', image=self.imDelete)
        self.btDelContrib.place(relx=1.0, y=0, x=-30, height=25, width=25)

        self.cbContribs = []
        self.edContribs = []  # actually, it will be the list of lists [[]]
        self.btPlotsContrib = []
        self.contribs_list = ['ConstBkg', 'LinearBkg', 'LorentzBkg',
            'Mie single', 'Mie LN', 'Spheroid',
            'Lorentz peak', 'Gauss peak']  # 'Au film', 'bst-3Au/glass']
        self.cbContribMats = []
        self.btContribDistribPlots = []

        # Fitting frame
        self.edExpFileName = ttk.Entry(self.fitting_frame, text='Exp. file name')
        self.edExpFileName.place(x=5, y=0, height=25, relwidth=0.8)

        self.btLoadExp = ttk.Button(self.fitting_frame, command=sup.btLoadExpClick,
                                    text='L', image=self.imLoad)
        self.btLoadExp.place(relx=1.0, x=-55, y=0, height=25, width=25)

        self.btPlotExp = ttk.Button(self.fitting_frame, command=sup.btPlotExpClick,
                                    text='P', image=self.imPlot)
        self.btPlotExp.place(relx=1.0, x=-30, y=0, height=25, width=25)

        self.btStartFit = ttk.Button(self.fitting_frame, command=sup.btStartFitClick,
                                     text='>', image=self.imPlay)
        self.btStartFit.place(x=5, y=30, height=25, width=25)

        self.btStopFit = ttk.Button(self.fitting_frame, command=sup.btStopFitClick,
                                    text='|', image=self.imStop)
        self.btStopFit.place(x=30, y=30, height=25, width=25)

        self.lbChiSq = ttk.Label(self.fitting_frame, text='ChiSq:')
        self.lbChiSq.place(x=60, y=35)

        self.btConstraints = ttk.Button(self.fitting_frame, command=sup.btConstraintsClick, text='Constraints...')
        self.btConstraints.place(relx=1, x=-100, y=30, height=25, width=95)

        self._create_menu(top)

        time_delta = time.time() - time_start  # in seconds
        if time_delta < self.splash_time:
            time.sleep(self.splash_time-time_delta)
        top.deiconify()
        splash.destroy()

    def configure_stvMaterial(self):
        self.stvMaterial.configure(columns='Col1')
        self.stvMaterial.heading('#0',text='MatID')
        self.stvMaterial.heading('#0',anchor='center')
        self.stvMaterial.column('#0',width='46')
        self.stvMaterial.column('#0',minwidth='20')
        self.stvMaterial.column('#0',stretch='1')
        self.stvMaterial.column('#0',anchor='w')
        self.stvMaterial.heading('Col1',text='Name')
        self.stvMaterial.heading('Col1',anchor='center')
        self.stvMaterial.column('Col1',width='150')
        self.stvMaterial.column('Col1',minwidth='20')
        self.stvMaterial.column('Col1',stretch='1')
        self.stvMaterial.column('Col1',anchor='w')

    def configure_stvSpheres(self):
        self.stvSpheres.configure(columns='Col1 Col2 Col3 Col4 Col5')
        self.stvSpheres.heading('#0', text='ID')
        self.stvSpheres.heading('#0', anchor='center')
        self.stvSpheres.column('#0', width='34')
        self.stvSpheres.column('#0', minwidth='20')
        self.stvSpheres.column('#0', stretch='1')
        self.stvSpheres.column('#0', anchor='w')
        self.stvSpheres.heading('Col1', text='R')
        self.stvSpheres.heading('Col1', anchor='center')
        self.stvSpheres.column('Col1', width='37')
        self.stvSpheres.column('Col1', minwidth='20')
        self.stvSpheres.column('Col1', stretch='1')
        self.stvSpheres.column('Col1', anchor='w')
        self.stvSpheres.heading('Col2', text='X')
        self.stvSpheres.heading('Col2', anchor='center')
        self.stvSpheres.column('Col2', width='31')
        self.stvSpheres.column('Col2', minwidth='20')
        self.stvSpheres.column('Col2', stretch='1')
        self.stvSpheres.column('Col2', anchor='w')
        self.stvSpheres.heading('Col3', text='Y')
        self.stvSpheres.heading('Col3', anchor='center')
        self.stvSpheres.column('Col3', width='34')
        self.stvSpheres.column('Col3', minwidth='20')
        self.stvSpheres.column('Col3', stretch='1')
        self.stvSpheres.column('Col3', anchor='w')
        self.stvSpheres.heading('Col4', text='Z')
        self.stvSpheres.heading('Col4', anchor='center')
        self.stvSpheres.column('Col4', width='34')
        self.stvSpheres.column('Col4', minwidth='20')
        self.stvSpheres.column('Col4', stretch='1')
        self.stvSpheres.column('Col4', anchor='w')
        self.stvSpheres.heading('Col5', text='mID')
        self.stvSpheres.heading('Col5', anchor='center')
        self.stvSpheres.column('Col5', width='32')
        self.stvSpheres.column('Col5', minwidth='20')
        self.stvSpheres.column('Col5', stretch='1')
        self.stvSpheres.column('Col5', anchor='w')

    def load_images(self):
        def tryload(fn):
            try:
                im = ImageTk.PhotoImage(file=os.path.normpath(os.path.join(os.path.dirname(__file__),'images', fn)))
            except Exception as err:
                print('Can not load %s\n%s' % (fn, err))
                return None
            return im
        self.imLoad    = tryload('folder_open_icon&16.png')
        self.imDelete  = tryload('delete_icon&16.png')
        self.imPlot    = tryload('chart_bar_icon&16.png')
        self.imPlot2   = tryload('chart_bar2_icon&16.png')
        self.imAdd     = tryload('sq_plus_icon&16.png')
        self.imSave    = tryload('save_icon&16.png')
        self.imRefresh = tryload('refresh_icon&16.png')
        self.imExport  = tryload('export_icon&16.png')
        self.imImport  = tryload('import_icon&16.png')
        self.imPlay    = tryload('playback_play_icon&16.png')
        self.imStop    = tryload('playback_stop_icon&16.png')
        self.imCalc    = tryload('cogs_icon&16.png')
        self.imEdit    = tryload('doc_edit_icon&16.png')
        self.imExit    = tryload('on-off_icon&16.png')
        self.imBrush   = tryload('brush_icon&16.png')
        self.imZoomIn  = tryload('round_plus_icon&16.png')
        self.imZoomOut = tryload('round_minus_icon&16.png')

    def __adjust_sash0(self, event):  # mysterious functions left from previous civilizations
        paned = event.widget
        pos = [220, 575, ]
        i = 0
        for sash in pos:
            paned.sashpos(i, sash)
            i += 1
        paned.unbind('<map>', self.__funcid0)
        del self.__funcid0

    def __adjust_sash1(self, event):
        paned = event.widget
        pos = [145, ]
        i = 0
        for sash in pos:
            paned.sashpos(i, sash)
            i += 1
        paned.unbind('<map>', self.__funcid1)
        del self.__funcid1

    def __adjust_sash2(self, event):
        paned = event.widget
        pos = [200, ]
        i = 0
        for sash in pos:
            paned.sashpos(i, sash)
            i += 1
        paned.unbind('<map>', self.__funcid2)
        del self.__funcid2

    def __adjust_sash3(self, event):
        paned = event.widget
        pos = [200, ]
        i = 0
        for sash in pos:
            paned.sashpos(i, sash)
            i += 1
        paned.unbind('<map>', self.__funcid3)
        del self.__funcid3

    def _create_menu(self, top):
        self.menubar = Menu(top)

        self.filemenu = Menu(self.menubar, tearoff=0)
        self.filemenu.add_command(label='Import spheres...', command=sup.btImportSpheres,
                                  image=self.imImport, compound='left')
        self.filemenu.add_command(label='Export spheres...', command=sup.btExportSpheres,
                                  image=self.imExport, compound='left')
        self.filemenu.add_separator()
        self.filemenu.add_command(label='Exit', command=sup.destroy_window,
                                  image=self.imExit, compound='left')
        self.menubar.add_cascade(label='File', menu=self.filemenu)

        self.matmenu = Menu(self.menubar, tearoff=0)
        self.matmenu.add_command(label='Add constant...', command=sup.btAddMatClick,
                                 image=self.imAdd, compound='left')
        self.matmenu.add_command(label='Load function...', command=sup.btLoadMatClick,
                                 image=self.imLoad, compound='left')
        self.matmenu.add_separator()
        self.matmenu.add_command(label='Delete selected', command=sup.btDelMatClick,
                                 image=self.imDelete, compound='left')
        self.matmenu.add_separator()
        self.matmenu.add_command(label='Plot selected', command=sup.btPlotMatClick,
                                 image=self.imPlot, compound='left')
        self.matmenu.add_separator()
        self.matmenu.add_command(label='Change view color...', command=sup.btChangeMatColClick,
                                 image=self.imBrush, compound='left')
        self.menubar.add_cascade(label='Materials', menu=self.matmenu)

        self.sphmenu = Menu(self.menubar, tearoff=0)
        self.sphmenu.add_command(label='Add...', command=sup.btAddSphClick,
                                 image=self.imAdd, compound='left')
        self.sphmenu.add_separator()
        self.sphmenu.add_command(label='Edit selected...', command=sup.btEditSphClick,
                                 image=self.imEdit, compound='left')
        self.sphmenu.add_command(label='Delete selected', command=sup.btDelSphClick,
                                 image=self.imDelete, compound='left')
        self.sphmenu.add_separator()
        self.sphmenu.add_command(label='Generate on mesh...', command=sup.btGenerateSpheresClick)
        self.menubar.add_cascade(label='Spheres', menu=self.sphmenu)

        self.viewmenu = Menu(self.menubar, tearoff=0)
        self.viewmenu.add_command(label='Zoom in',
            command=lambda : sup.mouse_wheel(type('', (), {'num':4, 'delta':0})()),
            image=self.imZoomIn, compound='left')
        self.viewmenu.add_command(label='Zoom out',
            command=lambda : sup.mouse_wheel(type('', (), {'num':5, 'delta':0})()),
            image=self.imZoomOut, compound='left')
        self.viewmenu.add_command(label='Reset view', command=sup.btPlotSphClick,
                                  image=self.imRefresh, compound='left')
        self.menubar.add_cascade(label='View', menu=self.viewmenu)

        self.opticsmenu = Menu(self.menubar, tearoff=0)
        self.opticsmenu.add_command(label='Calculate', command=sup.btCalcSpecClick,
                                    image=self.imCalc, compound='left')
        self.menubar.add_cascade(label='Spectrum', menu=self.opticsmenu)

        self.fittingmenu = Menu(self.menubar, tearoff=0)
        self.fittingmenu.add_command(label='Load experiment...', command=sup.btLoadExpClick,
                                     image=self.imLoad, compound='left')
        self.fittingmenu.add_separator()
        self.fittingmenu.add_command(label='Constraints...', command=sup.btConstraintsClick)
        self.fittingmenu.add_separator()
        self.fittingmenu.add_command(label='Start fit', command=sup.btStartFitClick,
                                     image=self.imPlay, compound='left')
        self.fittingmenu.add_command(label='Stop fit', command=sup.btStopFitClick,
                                     image=self.imStop, compound='left')
        self.menubar.add_cascade(label='Fitting', menu=self.fittingmenu)

        self.helpmenu = Menu(self.menubar, tearoff=0)
        self.helpmenu.add_command(label='About', command=sup.btAboutClick)
        self.menubar.add_cascade(label='Help', menu=self.helpmenu)
        # display the menu
        top.config(menu=self.menubar)




# The following code is added to facilitate the Scrolled widgets you specified.
class AutoScroll(object):
    '''Configure the scrollbars for a widget.'''

    def __init__(self, master):
        #  Rozen. Added the try-except clauses so that this class
        #  could be used for scrolled entry widget for which vertical
        #  scrolling is not supported. 5/7/14.
        try:
            vsb = ttk.Scrollbar(master, orient='vertical', command=self.yview)
        except:
            pass
        hsb = ttk.Scrollbar(master, orient='horizontal', command=self.xview)

        #self.configure(yscrollcommand=_autoscroll(vsb),
        #    xscrollcommand=_autoscroll(hsb))
        try:
            self.configure(yscrollcommand=self._autoscroll(vsb))
        except:
            pass
        self.configure(xscrollcommand=self._autoscroll(hsb))

        self.grid(column=0, row=0, sticky='nsew')
        try:
            vsb.grid(column=1, row=0, sticky='ns')
        except:
            pass
        hsb.grid(column=0, row=1, sticky='ew')

        master.grid_columnconfigure(0, weight=1)
        master.grid_rowconfigure(0, weight=1)

        # Copy geometry methods of master  (taken from ScrolledText.py)
        if py3:
            methods = Pack.__dict__.keys() | Grid.__dict__.keys() \
                  | Place.__dict__.keys()
        else:
            methods = Pack.__dict__.keys() + Grid.__dict__.keys() \
                  + Place.__dict__.keys()

        for meth in methods:
            if meth[0] != '_' and meth not in ('config', 'configure'):
                setattr(self, meth, getattr(master, meth))

    @staticmethod
    def _autoscroll(sbar):
        '''Hide and show scrollbar as needed.'''
        def wrapped(first, last):
            first, last = float(first), float(last)
            if first <= 0 and last >= 1:
                sbar.grid_remove()
            else:
                sbar.grid()
            sbar.set(first, last)
        return wrapped

    def __str__(self):
        return str(self.master)

def _create_container(func):
    '''Creates a ttk Frame with a given master, and use this new frame to
    place the scrollbars and the widget.'''
    def wrapped(cls, master, **kw):
        container = ttk.Frame(master)
        return func(cls, container, **kw)
    return wrapped

class ScrolledTreeView(AutoScroll, ttk.Treeview):
    '''A standard ttk Treeview widget with scrollbars that will
    automatically show/hide as needed.'''
    @_create_container
    def __init__(self, master, **kw):
        ttk.Treeview.__init__(self, master, **kw)
        AutoScroll.__init__(self, master)


if __name__ == '__main__':
    vp_start_gui()


